 {~ Measures a stellar image from Santa Barbara ST4 CCD camera ~}

program S4M;
{program Show_and_measure_CCD_ST4 by Jan Hollan using a unit from GRADA}
{measures a series of images}

{What to do yet: positions of oversaturated stars}

uses PG256,crt,dos,DbBridge,solar_ut,str_num,angles_o;

const cl=#13#10;
h0= cl+
'SC <ST4-file name> B# T# UT# DO# RI# RS# TO# MO MM# RD#'+cl
                                               +cl;
h1=
'Measures a stellar CCD frame from the ST4 camera:'+cl+
'                                 192 columns and 165 rows of byte.'+cl+
'The parameters (nome is obligatory) are as follows:'+cl;
h2=
'B   Bottom level to be subtracted from stars and to start the gray pallette'+cl+
'  (default 60% of the distribution function skipping the first and last row)'+cl+
'T   top level, just for the palette (default 0.9999 of that distrib. function)'+cl;
h3=
'UT  how many hours is the used time ahead of the UTC (default 1)'+cl+
'Do  approximate position of the central star in tenths of semi-side downward'+cl+
'Ri     -  "  -                                                    rightward'+cl;
h4=
'RS  radius for searching for a higher local maximum in pixels (default 4.0)'+cl+
'TO  threshold over Bottom for integrating around a local maximum (>0, def. 2)'+cl+
'MO  minimum over Bottom for a valid local maximum (>1, default 2)'+cl;
h5=
'MM  minimum multiple of the maximum pixel the integral should reach'+cl+
'   to declare the local maximum to be a star'+cl+
'RD  radius for a diaphragm for the final integration of brightness'+cl+
'   (default 3.5 pixels)'+cl+
    cl;
h6=
'When viewing the image after computation, the Bottom and Top level for the'+cl+
'gray scale may be given interactively as well, separated by a space.'+cl+
'Just the top (white) level may be changed by starting the line with a space.'+cl;
h7=
'A character as a first letter switches logarithmic/linear scale.'+cl
                                                                 +cl+
'Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1995,'+cl+
'(using a Turbo Pascal unit PG256 from the GRADA publishing house)'+cl;

 R: real=4.0;     {distance to search upwards for a larger local maximum -
                   must be >= 1.0 - to choose a right row of local maximum}
 OverBottom: byte=1;  {threshold over Bott_p percentile of the distribution
                   function for integrating around a local maximum;
                   can be made larger than 1 but smaller than ...Max}
 OverBottomMax: byte=2; {threshold over Bott_p percentile of the distribution
                   function for a valid local maximum;
                   no sense to have it less than 2}
 MoreThanMax: real=3.0; {the integral (visible brightness) is to be
                   this times larger than the excess of the Max pixel itself}
 t_UT : real=1.0;    {for the European winter time}
 right: shortint=0;  {approximate position of the central star
                   in tenths of a semi-side to the right}
 down: shortint=0; {... of a semi-side down from the center of the image}
 Bott_p = 0.60;   {percentil of distribution function to set the bottom level}
 Top_p = 0.9999;  {             -  "  -                             top level}
 R_dia: word=35;  {diameter of diaphragm in tenths of pixel size}
 test: boolean=false;

 top: byte=255;   {initial Top level for the image}
 bottom: byte=0;  {initial Top level for the image}
 Sc_r: real=2.64/165; {Horizontal scale 2.64 mm per 165 pixels}
 Sc_c: real=2.64/192; {Vertical                     192       }
 offs: word=$A000; {beginning of the video memory}
 Linear: boolean=false; {Showing the picture in a logarithmic scale at first}
 bits : array [0..7] of byte = (1,2,4,8,16,32,64,128);

 MaxBri  :word=0; {Brightness of the central star}
 Bri_Star:word=0; {Number of the -  "  -         within the local maxima}
 CSN: byte=0;     {        -       "       -     within the accepted stars}

type
 row = array[1..192] of byte;
 starRec= record
   SN,N_t:byte;
   row,col: word;
   sh_r,sh_c:shortint;
   br_t,SD,rad,ang,br_f,SD_f:real;
 end;
var
 N_SN, N_N_T, N_BR_T, N_SD, N_BR_F, N_SD_F, N_Radius, N_Angle, N_MT, N_MF,
 N_Name,N_MJD,N_row,N_col,N_Sh_r,N_Sh_c,N_NS,N_D_Ang,N_Sky,N_SD_S,N_St_P,
 N_Mx_z:
  byte;
 cc,k,m,Rbyte,up,NS,Hits,HitsR,LocMax,
  left_c,right_c,up_r,down_r: byte;
 ch: char;
 intln,RQ,Z,Sky_Av24,Sky_SD24,Sky_Dif,radius,angle,BriStar_r,BriStar_c,MJD,
 Mx_z: real;
 inp: file;
 Gray256_palette: RGB_Palette;
 buf: array[1..192] of byte;
 cbuf: array[1..192] of char absolute buf;
 within_star: array[1..165,0..23] of byte; {to have a boolean 1-bit array}
 UpRow_L_Max: array[1..5] of row;
 Row_Max,Row_L_Max: row;
 New_L_Max,New_R_Max: boolean;
 star_c,star_r,star_v: array[1..1000] of byte;
 star_shr,star_shc: array[1..1000] of shortint;
 star_bri, star_N: array[1..1000] of word;
 Star_list: text;
 distrib,Histogram: array[0..255] of word;
 NumRead,Wrong_pos,aux_i: integer;
 Sum_RM,Sum_CM,Sum_Q,Sum_E,Sum_Q_Sky,Sum_E_Sky,aux_l: longint;
 Sum_RM_R,Sum_CM_R,Sum_E_R:real;
 Su_RM,Su_CM: integer;
 Su_E,Sum_N,Sum_N_Sky, NC: word;
 aux_s:string;
 Bott_perc,Top_perc,N_stars,aux_w: word;
 No_Margin:boolean;
 Mean: byte;
 Sky_Av, Sky_SD, Sky_Av_SD: array[1..4] of real;
 Sky_Av_All, Sky_SD_All, Sky_Av_SD_All: real;
 StarPix_Num: array[1..4] of word;
 Star_Perc: real;
 Cat_Im: array[1..60] of starRec;
 Cat: array[1..255] of starRec;
 Dif_ang: array[1..59] of real;
 stars:text;
{ catalog: file of starRec;}

procedure help(message:string);
  begin
   PG256_end;
   if message<>'' then
    begin
     writeln(message);
     lowvideo;
     write(
 ' (press Q to quit, any other key to see the instructions)');
     ch:=readkey;
     if UpCase(ch)= 'Q' then halt;
    end
   else
    lowvideo;
   writeln(cl+
 'Usage:                                                    ');
   highvideo;
   write(h0);
   lowvideo;
   write(h1); write(h2); write(h3); write(h4); write(h5);
   write(
 ' (press Q to quit, any other key to see the last page of explanation)');
   ch:=readkey;
   if UpCase(ch)= 'Q' then halt;
   writeln(#13+h6);
   write(h7); halt;
  end;

procedure Stop(message:string);
begin
 PG256_end;
 TextColor(15);
 writeln(message);
 LowVideo;

 if Test then    {test output}
  begin
   assign(star_list,'s4m.log');
   rewrite(star_list);
   for m:=1 to 165 do
    begin
     writeln(star_list);
     for k:=0 to 23 do write(star_list,within_star[m,k]:3)
    end;
   close(star_list)
  end;

 halt
end;

Function Min1(a:integer):byte;
begin
 if a<1 then Min1:=1 else Min1:=a
end;

Function Max192(a:integer):byte;
begin
 if a>192 then Max192:=192 else if a<0 then Max192:=0 else Max192:=a
end;

procedure row_sum(row,col: word; dif_r: integer);
var st_r,st_c:byte;
    S_E,aux_r:real;

 function Inside:boolean;
 var r:word;
 begin
  r:=trunc(sqrt(sqr(k*10.0-col)+sqr(st_r*10.0-row)));
  if r<=R_dia then Inside:=true else Inside:=false
 end;

begin
 st_c:=round(col/10.0);
 st_r:=round(row/10.0);
 inc(st_r,dif_r);
 offs:=$A000+(st_r-1)*20;
 Move(Mem[offs:0],buf[1],192);

 k:=St_c;
 if not Inside then exit
 else
  begin
   aux_r:=buf[k]-Bottom-Sky_Dif;
   S_E := aux_r;
   inc(Sum_N);
   Sum_RM_R:=Sum_RM_R+aux_r*dif_r
  end;
 cc:=1;
 k:=st_c-1;
 while Inside do  {to the left}
  begin
   aux_r:=buf[k]-Bottom-Sky_Dif;
   Sum_CM_R:=Sum_CM_R-aux_r*cc;
   Sum_RM_R:=Sum_RM_R+aux_r*dif_r;
   S_E:=S_E+aux_r;
   inc(Sum_N);
   inc(cc);
   dec(k);
  end;
 cc:=1;
 k:=st_c+1;
 while Inside do  {to the right}
  begin
   aux_r:=buf[k]-Bottom-Sky_Dif;
   Sum_CM_R:=Sum_CM_R+aux_r*cc;
   Sum_RM_R:=Sum_RM_R+aux_r*dif_r;
   S_E:=S_E+aux_r;
   inc(Sum_N);
   inc(cc);
   inc(k);
  end;
 Sum_E_R:=Sum_E_R+S_E;
end;


function row_mean(st_r,st_c: byte; dif_r: integer; var S_E: word):boolean;
var No_RMargin,No_LMargin,No_BTMargin: boolean;

 procedure Within;
 begin
  inc(within_star[st_r, (k-1) shr 3],bits[k - (k-1) shr 3 shl 3 -1]);
 end;

begin
 row_mean:= true;
 No_BTMargin:= true;
 No_LMargin:= true;
 No_RMargin:= true;
 inc(st_r,dif_r);
 offs:=$A000+(st_r-1)*20;
 Move(Mem[offs:0],buf[1],192);

 if buf[st_c]<= bottom+OverBottom then begin S_E:=0; exit end
 else
  begin
   k:=st_c;
   if (st_r=2) or (st_r=165) then No_BTMargin:=false;
   aux_l:=buf[k]-Bottom;
   S_E := aux_l;
   Within;
   inc(Sum_N);
   inc(Sum_RM,aux_l*dif_r)
  end;
 cc:=1;
 k:=st_c-1;
 if k=0 then No_LMargin:=false;
 while No_LMargin and (buf[k]>Bottom+OverBottom) do  {to the left}
  begin
   aux_l:=buf[k]-Bottom;
   dec(Sum_CM,aux_l*cc);
   inc(Sum_RM,aux_l*dif_r);
   inc(S_E, aux_l);
   Within;
   inc(Sum_N);
   inc(cc);
   dec(k);
   if k=0 then No_LMargin:=false;
  end;
 cc:=1;
 k:=st_c+1;
 if k=193 then No_RMargin:=false;
 while No_RMargin and (buf[k]>Bottom+OverBottom) do  {to the right}
  begin
   aux_l:=buf[k]-Bottom;
   inc(Sum_CM,aux_l*cc);
   inc(Sum_RM,aux_l*dif_r);
   inc(S_E, aux_l);
   Within;
   inc(Sum_N);
   inc(cc);
   inc(k);
   if k=193 then No_RMargin:=false;
  end;
 inc(Sum_E,S_E);
 row_mean:= {No_RMargin and  No_LMargin and}  No_BTMargin;
 No_Margin:= No_Margin and No_RMargin and  No_LMargin and No_BTMargin
end;

procedure Sky(left,right,upper,base,Num:byte);
begin
 Sum_N:=0;
 Sum_E:=0;
 Sum_Q:=0;
  for m:=upper to base do
   begin
    offs:=$A000+(m-1)*20;
    Move(Mem[offs:0],buf[1],192);
    for k:=left to right  do
     if (within_star[m, (k-1) shr 3] and bits[k- (k-1) shr 3 shl 3 -1]) = 0
     then
      begin
       inc(Sum_N);
       aux_l:=buf[k];
       inc(Sum_E,aux_l);
       inc(Sum_Q,sqr(aux_l));
      end
   end;
  Sky_Av[Num]:=Sum_E / Sum_N;
  Sky_SD[Num]:=sqrt((Sum_Q - Sum_E*Sky_Av[Num])/Sum_N);
  Sky_Av_SD[Num]:=Sky_SD[Num]/sqrt(Sum_N);
  StarPix_Num[Num]:= (1+right-left)*(1+base-upper) - Sum_N;
  inc(Sum_E_Sky,Sum_E);
  inc(Sum_N_Sky,Sum_N);
  inc(Sum_Q_Sky,Sum_Q);
end;

procedure NewStar;
begin
     with Cat_Im[k] do
     begin
      inc(NC);
      SN:=NC;
      DbWrNum(N_SN,SN);
      DbWrNum(N_N_t,N_t);
      DbWrNum(N_Br_t,Br_t);
      DbWrNum(N_SD,sd);
      DbWrNum(N_Br_f,Br_f);
      DbWrNum(N_SD_f,SD_f);
      DbWrNum(N_Radius,rad);
      DbWrNum(N_Angle,ang);
      DbWrNum(N_MT,1);
      DbWrNum(N_MF,1);
      if DbWrite(0,rec) then;
     end
end;


begin
 highvideo;

 {processing parameters}
 IF pARAMCOUNT<1 THEN help('');
 FillChar(Gray256_palette,3*256,0);
 FillChar(Histogram,512,0);
 If ParamCount>1 then for k:=2 to paramcount do
  begin
   aux_s:=paramstr(k);
   if (aux_s[1]='/') or (aux_s[1]='-') then aux_s:=copy(aux_s,2,length(aux_s)-1);
   aux_s[1]:=UpCase(aux_s[1]);
   aux_s[2]:=UpCase(aux_s[2]);
    case aux_s[1] of
     '?' : help('');
     'B' : Bottom:=round(par_num(aux_s));
     'T' : if aux_s[2]='O' then
            begin
             {the O is skipped}
             aux_s:=copy(aux_s,2,length(aux_s)-1);
             OverBottom:=round(par_num(aux_s));
            end
           else
            if aux_s[2]='E' then Test:=true
            else Top:=round(par_num(aux_s));
     'U' : begin
            {the T is skipped}
            aux_s:=copy(aux_s,2,length(aux_s)-1);
            UT:=par_num(aux_s);
           end;
     'D' : begin
            {the o is skipped}
            aux_s:=copy(aux_s,2,length(aux_s)-1);
            Down:=round(par_num(aux_s));
           end;
     'M' : begin
             aux_s:=copy(aux_s,2,length(aux_s)-1);
             case aux_s[1] of
              'O':  OverBottomMax:=round(par_num(aux_s));
              'M':  MoreThanMax:=par_num(aux_s);
             end;
           end;
     'R' : begin
             aux_s:=copy(aux_s,2,length(aux_s)-1);
             case aux_s[1] of
              'I':  Right:=round(par_num(aux_s));
              'S':  R:=par_num(aux_s);
              'D':  R_dia:=round(10*par_num(aux_s));
             end;
           end;
     end;
  end;

 up_r:=Min1(trunc(163*(0.5+down/20-0.2)));
 down_r:=Max192(trunc(163*(0.5+down/20+0.2)));
 left_c:=Min1(trunc(192*(0.5+right/20-0.2)));
 right_c:=Max192(trunc(192*(0.5+right/20+0.2)));

 {interval of pixel values per one gray level - checking if top>bottom}
 intln:=(top-bottom)/62;
 if intln <= 0 then
  begin
   writeln('The top level is not greater than the bottom one.');
   halt
  end;

 {reading the image}
{$I-}
 assign(inp,paramstr(1));
 reset(inp,192);
 if IORESULT<>0 THEN
  begin
   aux_s:=ParamStr(1);
   if not ((aux_s[1]='/') or (aux_s[1]='?')) then
    writeln('There is no file >',ParamStr(1),'< in this directory!');
   help('')
  end;
{$I+}

 {setting the gray palette and 320x200 graphic screen}
 if bottom>0 then for cc:=1 to bottom do with Gray256_palette[cc] do G:=0;
 for cc:=top to 255 do with Gray256_palette[cc] do G:=63;
 m:=bottom+1; k:=top-1;
 for cc:=m to k do with Gray256_palette[cc] do G:=1+trunc((cc-bottom)/intln);
 PG256_Start;
 Set256_Palette(Gray256_palette);

 {putting the image to the 320x200 screen}
 BlockRead(inp,buf,1,NumRead);
 Move(buf[1], Mem[offs:0], 192);
 inc(offs,20);
 for m:=2 to 164 do
  begin
   BlockRead(inp,buf,1,NumRead);
   Move(buf[1], Mem[offs:0], 192);
   inc(offs,20);
   for k:=1 to 192 do inc(Histogram[buf[k]]);
  end;
 BlockRead(inp,buf,1,NumRead);  {last image line}
 Move(buf[1], Mem[offs:0], 192);


 {painting the gray scale and a rule showing its values}
 for cc:=0 to 255 do for k:=167 to 177 do Set256_Pixel(cc,k,cc);
 for cc:=0 to 25 do for k:=178 to 188 do Set256_Pixel(10*cc,k,255);
 for cc:=0 to 5 do for k:=189 to 199 do Set256_Pixel(50*cc,k,255);

 TextColor(200);
 GoToXY(26,1);
 Distrib:=Histogram;
 for m:=1 to 255 do inc(Distrib[m],Distrib[m-1]);

 Aux_s:='';
 Bott_perc:=trunc(163*192*Bott_P);
                             {not 165* as the top and bottom lines are bad}
 Top_perc:=trunc(163*192*Top_p);

 {counting the bottom and top level from the distribution of pixel values}
 m:=0; k:=0;
 while k=0 do
  begin
   if Distrib[m]>Bott_perc then k:=m;
   inc(m)
  end;
 if bottom=0 then Bottom:=k;
 k:=top;
 while (k=255) and (m<254) do
  begin
   inc(m);
   if Distrib[m]> Top_perc then k:=m
  end;
 if top=255 then Top:=k;


 {finding the local maxima - the first contaminated line is excluded}
 N_Stars:=0;
 Rbyte:=Trunc(R);
 RQ:=sqr(R);
 fillchar(UpRow_L_max,5*192,0);
 fillchar(within_star,24*165,0);
 for m:=2 to 165 do
  begin
   offs:=$A000+(m-1)*20;
   Move(Mem[offs:0],buf[1],192);
   fillchar(row_max,192,0);
   Row_L_Max:=Row_Max;
   k:=1;
   while k<191 do
    begin
     while (buf[k+1]>buf[k]) and (k<191) do inc(k);
     if buf[k+1]>buf[k] then inc(k);   {to reach the right margin}
     LocMax:=buf[k];
     if LocMax>Bottom+OverBottomMax then
      begin
       New_R_max:=true; New_L_Max:=true;
       for cc:=Min1(k-Rbyte) to k-1 do
        if buf[cc]>LocMax then     {a larger maximum withim Rbyte before}
         begin New_L_max:=false; New_R_Max:=false end;
       up:=1;
       while New_L_Max and (up<=Rbyte) and (m-up>1) do
        begin
         dec(offs,20);
         Move(Mem[offs:0],buf[1],192);
         cc:=trunc(sqrt(RQ-up*up));
         for cc:=Min1(k-cc) to Max192(k+cc) do
          if buf[cc]>LocMax then
           New_L_max:=false;           {a larger maximum within Rbyte above}
         inc(up)
        end;
       if New_L_Max then
        begin
         Row_L_Max[k]:=LocMax;
         for cc:=Min1(k-Rbyte) to k-1 do Row_L_Max[cc]:=0;
         up:=1;
         while New_L_Max and (up<=Rbyte) and (m-up>1) do
          begin
           cc:=trunc(sqrt(RQ-up*up));
           for cc:=Min1(k-cc) to Max192(k+cc) do UpRow_L_Max[up,cc]:=0;
           inc(up)
          end;
        end;
       if New_R_Max then
        begin
         Row_Max[k]:=LocMax;
         for cc:=Min1(k-Rbyte) to k-1 do Row_Max[cc]:=0;
        end;
      end;
     offs:=$A000+(m-1)*20;
     Move(Mem[offs:0],buf[1],192);
     while (k<191) and (buf[k+1]<=buf[k]) do inc(k);
    end;
   for cc:=1 to 192 do if UpRow_L_Max[Rbyte,cc]>0 then
    begin
     inc(N_Stars);
     star_c[N_Stars]:=cc;
     star_r[n_Stars]:=m-Rbyte;
     star_v[n_stars]:=UpRow_L_Max[Rbyte,cc]-Bottom
    end;
   if Rbyte>1 then
    for up:=Rbyte downto 2 do UpRow_L_Max[up]:=UpRow_L_Max[up-1];
   UpRow_L_Max[1]:=Row_L_Max;
  end;
 for up:=1 to Rbyte do for cc:=1 to 192 do
  if UpRow_L_Max[up,cc]>0 then
   begin
    inc(N_Stars);
    star_c[N_Stars]:=cc;
    star_r[n_Stars]:=165-up+1;
    star_v[n_stars]:=UpRow_L_Max[up,cc]-Bottom
   end;
 {end of finding the local maxima}


 {processing the found local maxima (stars)}
 if N_Stars>0 then
 begin

  {measuring the possible stars first time - till their margins}
  MaxBri:=0;
  NS:=0;
  for aux_w:=1 to N_Stars do
   begin
    star_Bri[aux_w]:= 0;
    No_Margin:=true;
    Sum_RM:=0;
    Sum_CM:=0;
    Sum_N:=0;
    Sum_E:=0;
    if row_mean(star_r[aux_w],star_c[aux_w],0,Su_E) then;
    up:=1;
    if star_r[aux_w]>2 then
     begin
      while row_mean(star_r[aux_w],star_c[aux_w],-up,Su_E) and (Su_E>0) do
       inc(up);
      up:=1
     end;
    if star_r[aux_w]<165 then
     while row_mean(star_r[aux_w],star_c[aux_w],up,Su_E) and (Su_E>0) do
      inc(up);
    if (Sum_E > star_v[aux_w]*MoreThanMax) and No_Margin then
     begin
      inc(NS);
      star_shc[aux_w]:=(10*Sum_CM) div Sum_E;
      star_shr[aux_w]:=(10*Sum_RM) div Sum_E;
      star_Bri[aux_w]:= Sum_E;
      star_N[aux_w]:= Sum_N;
      if (Sum_E>MaxBri) and (star_r[aux_w] in [up_r .. down_r]) and
                            (star_c[aux_w] in [left_c .. right_c]) then
       begin
        Bri_star:=aux_w;
        MaxBri:=Sum_E
       end
     end
    else if (Sum_E <= star_v[aux_w]*MoreThanMax) then star_v[aux_w]:=0;
    if (abs(star_shc[aux_w])>19) or (abs(star_shr[aux_w])>16)
    or ((star_v[aux_w]<2*OverBottomMax) and (Sum_E>star_v[aux_w]*10)) then
     star_v[aux_w]:=0
   end;
  {end of measuring the stars first time}

  {a "central" star}
  if MaxBri>0 then
   begin
    BriStar_r:=star_r[Bri_Star]+star_shr[Bri_Star]/10;
    BriStar_c:=star_c[Bri_Star]+star_shc[Bri_Star]/10;
   end;
 end;

  {measuring the sky}
  Sum_E_Sky:=0;
  Sum_N_Sky:=0;
  Sum_Q_Sky:=0;
  Sky( 1, 96, 2, 83,1);
  Sky( 1, 96,84,164,2);
  Sky(97,192,84,164,3);
  Sky(97,192, 2, 83,4);
  Sky_Av_All:=Sum_E_Sky / Sum_N_Sky;
  Sky_SD_All:=sqrt((Sum_Q_Sky - Sum_E_Sky*Sky_Av_All)/Sum_N_Sky);
  Sky_Av_SD_All:=Sky_SD_All/sqrt(Sum_N_Sky);
  Star_Perc:= 100 - Sum_N_Sky/(1.92*163);
  Sky_Dif:=Sky_Av_All-Bottom;

  {a beginning of the report (*.lst) file on the image}
  BlockRead(inp,buf,1,NumRead);  {image description}
  close(inp);
  assign(star_list,copy(Paramstr(1),1,pos('.',Paramstr(1)))+'lst');
  rewrite(star_list);
  write(Star_list,
'Results of the file ',paramstr(1));
  if length(paramstr(1))<12 then
   for k:=1 to 12-length(paramstr(1)) do write(Star_list,' ');
  writeln(Star_list,'  exp./0.01s   fc.l./"  ap.ar/sq"   cal.f.');
  write(Star_list,
'                    -----------');
  for k:=81 to 90 do write(Star_list,cbuf[k]); write(Star_list,'   ');
  for k:=91 to 99 do write(Star_list,cbuf[k]); write(Star_list,'  ');
  for k:=100 to 109 do write(Star_list,cbuf[k]); write(Star_list,'  ');
  for k:=110 to 119 do write(Star_list,cbuf[k]); write(Star_list,'  ');
  writeln(Star_list);
  for k:=2 to 2+76 do write(Star_list,cbuf[k]);
  writeln(Star_list);
  for k:=81+40 to 192 do
   if buf[k]>0 then write(Star_list,cbuf[k])
   else write(Star_list,' ');

  {Julian date}
  val(cbuf[16]+cbuf[17],m,Wrong_pos);
  val(cbuf[19]+cbuf[20],k,Wrong_pos);
  val(cbuf[22]+cbuf[23],cc,Wrong_pos);
   {writeln(Star_list,cl+'YY MM DD = ',cc:2,m:2,k:2);}

  MJD:=JD0(cc+1900,m,k)-2400000.5;
  val(cbuf[28]+cbuf[29],cc,Wrong_pos);
  val(cbuf[31]+cbuf[32],m,Wrong_pos);
  val(cbuf[34]+cbuf[35],k,Wrong_pos);
   {writeln(Star_list,cl+'hh mm ss = ',cc:2,m:2,k:2);}
  MJD:=MJD+((k/60+m)/60+cc-t_UT)/24;
  writeln(Star_list,cl+'Modified Julian Date = ',MJD:10:4);

 if NS>0 then    {if some local maxima appear to be stars}
 begin
  {"writing" the final stars into a local catalog of the image}
  NS:=0;
  for aux_w:=1 to N_Stars do
   if (star_v[aux_w]>0) and (star_Bri[aux_w]>0) then
    begin
      if (aux_w<>Bri_Star) and (MaxBri>0) then
       C2_to_Pd((-BriStar_c + star_c[aux_w]+star_shc[aux_w]/10) * Sc_r,
               (  BriStar_r - star_r[aux_w]-star_shr[aux_w]/10) * Sc_c,
               radius,angle)
      else
       begin
        radius:=0;
        angle:=0
       end;
      inc(NS);

      with cat_im[NS] do
       begin
        row:=round(10*star_r[aux_w]+star_shr[aux_w]);
        col:=round(10*star_c[aux_w]+star_shc[aux_w]);
        SN:=0;
        br_t:=star_bri[aux_w]-Sky_Dif*star_N[aux_w];
        SD:=Sky_SD_All*sqrt(star_N[aux_w]);
        N_t:=star_N[aux_w];
        rad:=radius;
        ang:=angle;
        if (0.0+row-R_Dia>10) and (row+R_Dia<1650)
         and (col+R_Dia<1930)  and (0.0+col-R_Dia>0) then
         begin
          {measuring the star the second time -
              in a diaphragm of the same size for all stars}
          No_Margin:=true;
          Sum_RM_R:=0;
          Sum_CM_R:=0;
          Sum_E_R:=0;
          Sum_N:=0;
          for aux_i:=-(R_Dia div 10)-1 to (R_Dia div 10)+1 do
           row_sum(row,col,aux_i);
          sh_c:=round(10*(Sum_CM_R/Sum_E_R -(col/10.0-round(col/10.0))));
          sh_r:=round(10*(Sum_RM_R/Sum_E_R -(row/10.0-round(row/10.0))));
          Br_f:=Sum_E_R/Sum_N;
          SD_f:=Sky_SD_All/sqrt(Sum_N)
         end
        else
         begin
          sh_c:=0;
          sh_r:=0;
          Br_f:=0;
          SD_f:=0
         end
       end
    end;


  {working with catalogs of stars}
  if MaxBri>0 then
  begin
  if DbInit('Star_Cat.dbf') then
   begin
    N_SN:=DbFieldNum('SN');
    N_N_T:=DbFieldNum('N_T');
    N_BR_T:=DbFieldNum('BR_T');
    N_SD:=DbFieldNum('SD');
    N_BR_F:=DbFieldNum('BR_F');
    N_SD_F:=DbFieldNum('SD_F');
    N_Radius:=DbFieldNum('RADIUS');
    N_Angle:=DbFieldNum('ANGLE');
    N_MT:=DbFieldNum('MT');
    N_MF:=DbFieldNum('MF');

   {identifying the image catalog with the main catalog,
            where the first record is the central star}
    NC:=1;
    while DbRead(NC,rec) do with Cat[NC] do
     begin
      val(rec[N_Sn],SN,Wrong_Pos);
      val(rec[N_Radius],RAD,Wrong_Pos);
      val(rec[N_Angle],ANG,Wrong_Pos);
      inc(NC);
     end;
    dec(NC);
    repeat
     Hits:=0; HitsR:=0;
     for k:=1 to NS do with Cat_Im[k] do for m:=2 to NC do
      if abs(rad-Cat[m].rad)<0.02 then
       if (     abs(ang-Cat[m].ang) *rad<1)
       or ((360-abs(ang-Cat[m].ang))*rad<1) then
        begin
         SN:=Cat[m].SN;
         inc(Hits)
        end
       else
        if (abs(180-abs(ang-Cat[m].ang))*rad<1) and (SN=0)
         then SN:=Cat[1].SN
        else
         begin
          inc(HitsR);
          Dif_Ang[HitsR]:=ang-Cat[m].ang
         end;
     m:=0;
     for k:=1 to NS do if Cat_Im[k].SN=1 then m:=k; {a reversed pair found}
     if Hits=0 then
      if m=0 then {no reversed pair}
       begin
       {No identification, perhaps rotated.}
        if HitsR>0 then for k:=1 to HitsR do
         writeln(Star_list,Dif_Ang[k]);
        close(star_list);
        stop('Rotated? See the *.lst file containing the differences of angles.');
       end
      else
       begin
       {a new central star}
        for k:=1 to NS do with Cat_Im[k] do
         begin
          if k<>m then
           C2_to_Pd((-Cat_Im[m].col + col) * Sc_r,
                    ( Cat_Im[m].row - row) * Sc_c, rad,ang)
          else
           begin
            rad:=0;
            ang:=0
           end;
          SN:=0
         end
       end
     else {if some hits}
      begin
       for k:=1 to NS do
        if Cat_Im[k].rad=0 then begin Cat_Im[k].SN:=1; CSN:=k end;
       for k:=1 to NS do
        if Cat_Im[k].SN>0 then with cat_im[k] do  {changing the former items}
         begin
          if DbRead(SN,rec) then;
          val(rec[N_MT],m,Wrong_pos);
          val(rec[N_MF],aux_i,Wrong_pos);
          rec[N_N_t]:=SI(3,round((N_S(rec[N_N_t])*m+N_t)/(m+1)));
          if m=1 then
           begin
            DbWrNum(N_SD,abs(N_S(rec[N_Br_t])-Br_t)/sqrt(2));
            DbWrNum(N_SD_f,abs(N_S(rec[N_Br_f])-Br_f)/sqrt(2))
           end
          else
           begin
            DbWrNum(N_SD,sqrt(sqr(N_S(rec[N_SD]))*(m-1)/m
                               +sqr(N_S(rec[N_Br_t])-Br_t)/(m+1)));
            if SD_f>0 then
             DbWrNum(N_SD_f,sqrt(sqr(N_S(rec[N_SD_f]))*(aux_i-1)/m
                               +sqr(N_S(rec[N_Br_f])-Br_f)/(aux_i+1)))
           end;
          DbWrNum(N_Br_t,round((N_S(rec[N_Br_t])*m+Br_t)/(m+1)));
          DbWrNum(N_Radius,(N_S(rec[N_Radius])*m+rad)/(m+1));
          DbWrNum(N_Angle,(N_S(rec[N_Angle])*m+ang)/(m+1));
          DbWrNum(N_MT,m+1);
          if SD_f>0 then
           begin
            DbWrNum(N_Br_f,(N_S(rec[N_Br_f])*m+Br_f)/(aux_i+1));
            DbWrNum(N_MF,aux_i+1)
           end;
          if DbWrite(SN,rec) then;
         end;
       for k:=1 to NS do
        if Cat_Im[k].SN=0 then NewStar;  {appending new stars}
      end
    until Hits>0
   end
   {of identifying the image catalog with the main catalog}

  else
   begin
   {creating such a catalog}
    DbStruDescr('Star_Ca_.dbf');
    DbDescField('SN','N',3,0,'Star Number','');
     N_SN:=  DbNuRec;
    DbDescField('N_T','N',3,0,
'Number of pixels above threshold','');
     N_N_T:= DbNuRec;
    DbDescField('BR_T','N',4,0,'Sum above sky level...',
'... in these pixels above threshold -- like Brightness_Total');
     N_BR_T:=DbNuRec;
    DbDescField('SD' ,'N',2,0,'Standard Deviation of BR_T','');
     N_SD:=  DbNuRec;
    DbDescField('BR_F','N',6,2,'Sum above sky level in ...',
'Sum of values above the sky level in a diaphragm with radius of 3.5 pixels'+
' --  Brightness in this diaphragm');
     N_BR_F:=DbNuRec;
    DbDescField('SD_F','N',4,2,
'Standard Deviation of BR_F','');
     N_SD_F:=DbNuRec;
    DbDescField('RADIUS','N',5,3,
'Distance from the central star','in decipixels');
     N_Radius:=DbNuRec;
    DbDescField('ANGLE' ,'N',5,1,
'Position angle','Right from the central star = 0, up = 90,... ');
     N_Angle:=DbNuRec;
    DbDescField('MT','N',2,0,'Number of Measurements of Br_T','');
     N_MT:=  DbNuRec;
    DbDescField('MF','N',2,0,'Number of Measurements of Br_F','');
     N_MF:=  DbNuRec;
    DbClose;


    if not DbNewFrom('Star_Cat.dbf') then
     help('Wrong Star_cat.dbf definition!');
    NC:=0;
    for k:=1 to NS do if Cat_Im[k].rad=0 then begin NewStar; CSN:=k end;
                                        {there is only 1 such star}
    for k:=1 to NS do if Cat_Im[k].rad<>0 then NewStar;
   end;
  {end of working with catalogs of stars}
  DbClose;
  end;

  {appending a list of all star measurements}
  if not DbInit('Stars.dbf') then
   begin
    DbStruDescr('Stars_.dbf');
    DbDescField('NAME','C',12,0,'Name of the file','');
     N_Name:=DbNuRec;
    DbDescField('MJD' ,'N',11,5,'its Modified Julian date','');
     N_MJD:=DbNuRec;
    DbDescField('SN','N',3,0,'Star Number','');
     N_SN:=DbNuRec;
    DbDescField('ROW','N',4,0,'Position of the star','in tenths of row');
     N_Row:=DbNuRec;
    DbDescField('COL','N',4,0,'Position of the star','in tenths of column');
     N_Col:=DbNuRec;
    DbDescField('N_T','N',3,0,'Number of pixels above threshold','');
     N_N_T:=DbNuRec;
    DbDescField('BR_T','N',4,0,
 'Sum of values above sky level...','... in these pixels - like Brightness_Total');
     N_Br_t:=DbNuRec;
    DbDescField('SD' ,'N',2,0,'Standard Deviation of BR_T','');
     N_SD:=DbNuRec;
    DbDescField('BR_F','N',6,2,
 'Sum of values above the sky...','... level in a diaphragm with radius of 3.5 pixels');
     N_Br_f:=DbNuRec;
    DbDescField('SD_F','N',4,2,'Standard Deviation of...',
 '... this Brightness (i.e., of BR_F)');
     N_SD_F:=DbNuRec;
    DbDescField('SH_R','N',3,0,
 'horizontal Shift of centre','against the brightest pixel');
     N_Sh_r:=DbNuRec;
    DbDescField('SH_C','N',3,0,
 'vertical Shift of centre','against the brightest pixel');
     N_Sh_c:=DbNuRec;
    DbDescField('RADIUS','N',5,3,
 'Distance from the central star','in decipixels');
     N_Radius:=DbNuRec;
    DbDescField('ANGLE' ,'N',5,1,
 'Position angle','right from the central star = 0, up = 90');
     N_Angle:=DbNuRec;
    DbClose;

    If not DbNewFrom('Stars.dbf') then Help('Wrong Stars.dbf definition!');
   end
  else
   begin
    N_Name:=DbFieldNum('NAME');
    N_MJD:=DbFieldNum('MJD');
    N_SN:=DbFieldNum('SN');
    N_row:=DbFieldNum('ROW');
    N_col:=DbFieldNum('COL');
    N_BR_T:=DbFieldNum('BR_T');
    N_SD:=DbFieldNum('SD');
    N_N_T:=DbFieldNum('N_T');
    N_BR_F:=DbFieldNum('BR_F');
    N_SD_F:=DbFieldNum('SD_F');
    N_Sh_r:=DbFieldNum('SH_R');
    N_Sh_c:=DbFieldNum('SH_C');
    N_Radius:=DbFieldNum('RADIUS');
    N_Angle:=DbFieldNum('ANGLE')
   end;

  rec[N_Name]:=Paramstr(1);
  DbWrNum(N_MJD,MJD);
  for k:=1 to NS do with Cat_Im[k] do
    begin
     DbWrNum(N_SN,SN);
     DbWrNum(N_Row,row);
     DbWrNum(N_Col,col);
     DbWrNum(N_Br_t,Br_t);
     DbWrNum(N_SD,sd);
     DbWrNum(N_N_t,N_t);
     DbWrNum(N_Br_f,Br_f);
     DbWrNum(N_SD_f,sd_f);
     DbWrNum(N_Sh_r,Sh_r);
     DbWrNum(N_Sh_c,Sh_c);
     DbWrNum(N_Radius,rad);
     DbWrNum(N_Angle,ang);
     if DbWrite(0,rec) then
    end;
  DbClose;

  {continuing the report (*.lst) file:}
  {the star list}
  writeln(Star_list,cl+cl+
'Star list for bottom=',Bottom:3,
                 ', maxima at least ',OverBottomMax+1,' over it,'+cl+
'                              summing till ',OverBottom+1,' over it,'+cl+
'          letting just stars with sums >',MoreThanMax:4:1,' more then Maximum'+cl+
'          and not allowing the maxima to be closer than ',R:4:1,' to each other.');
  writeln(Star_list,cl+
'row  col. m-b. shr shc   sum  SD  N  r_B  angle');
  k:=0;
  for aux_w:=1 to N_Stars do
   if star_v[aux_w]>0 then
    begin
     write(star_list,star_r[aux_w]:3,star_c[aux_w]:5,star_v[aux_w]:4);
     if star_Bri[aux_w]>0 then
      begin
       inc(k);
       with cat_im[k] do
        begin
         writeln(star_list,star_shr[aux_w]:4,star_shc[aux_w]:4,
                Br_t:7:0,SD:4:0, N_t:4,rad:6:3,ang:6:1,SN:3);
         if SN=1 then
          begin
           Set256_Pixel(star_c[aux_w],star_r[aux_w]-1,255);
           Set256_Pixel(star_c[aux_w]-1,star_r[aux_w],255);
          end
        end
      end
     else
      writeln(star_list);
     Set256_Pixel(star_c[aux_w]-1,star_r[aux_w]-1,255)
    end;
  {end of writing star list}
 end;

  {writing an information concerning the sky}
  Mx_z:=0;
  for cc:=2 to 3 do
   begin
    Z:=abs(Sky_Av[cc]-Sky_Av[cc+1])
       / sqrt(sqr(Sky_Av_SD[cc])+sqr(Sky_Av_SD[cc+1]));
    if Z>3 then writeln(Star_list,cl+
'The sky in quadrants ',cc,' and ',cc+1,' differs by',Z:5:1,' SD of average!');
    if Z>Mx_z then Mx_z:=Z;
   end;

  Z:=abs(Sky_Av[1]-Sky_Av[3])/sqrt(sqr(Sky_Av_SD[1])+sqr(Sky_Av_SD[3]));
  if Z>3 then writeln(Star_list,cl+
'The hot quadrant differs from the opposite one by',Z:5:1,' SD of average!');

  if Z>Mx_z then Mx_z:=Z;
  writeln(Star_list,cl+
'Ouadrants: (the hot one=1) 1 | 4'+cl+
'                           -----'+cl+
'                           2 | 3');
  writeln(Star_list,cl+
'          Average   its SD  pixel SD   N of star pixels');
  for cc:=1 to 4 do
   writeln(Star_list,
cc:3, Sky_Av[cc]:13:2,Sky_Av_SD[cc]:9:3, Sky_SD[cc]:9:2, StarPix_Num[cc]:8);

   writeln(Star_list,
'All', Sky_Av_All:13:2,Sky_Av_SD_All:9:3, Sky_SD_All:9:2, Star_Perc:8:1,'%');

  {appending a catalog of images}
  if not DbInit('Images.dbf') then
   begin
    DbStruDescr('Images_.dbf');
     DbDescField('NAME','C',12,0,'Name of the file','');
      N_Name:=DbNuRec;
     DbDescField('MJD' ,'N',11,5,'its Modified Julian date','');
      N_MJD:=DbNuRec;
     DbDescField('NS','N',3,0,'Number of Stars','');
      N_NS:=DbNuRec;
     DbDescField('ROW','N',4,0,'Position of the central star','in tenths of row');
      N_Row:=DbNuRec;
     DbDescField('COL','N',4,0,'Position of the central star','in tenths of column');
      N_Col:=DbNuRec;
     DbDescField('D_ANG','N',5,2,'Rotation against the fist image','degrees');
      N_D_Ang:=DbNuRec;
     DbDescField('SKY' ,'N',6,2,'Average level of Sky','without stars');
      N_SKY:=DbNuRec;
     DbDescField('SD_S' ,'N',3,1,'Standard deviation of Sky level','');
      N_SD_S:=DbNuRec;
     DbDescField('ST_P','N',5,1,'Per cent of image filled...','... with stars');
      N_St_P:=DbNuRec;
     DbDescField('MX_Z','N',4,1,
 'Maximum z: difference of sky...','... in two quadrants / by SD of this difference');
      N_Mx_z:=DbNuRec;
    DbClose;

    If not DbNewFrom('Images.dbf') then Help('Wrong Stars.dbf definition!');
   end
  else
   begin
    N_Name:=DbFieldNum('NAME');
    N_MJD:=DbFieldNum('MJD');
    N_NS:=DbFieldNum('NS');
    N_row:=DbFieldNum('ROW');
    N_col:=DbFieldNum('COL');
    N_D_Ang:=DbFieldNum('D_ANG');
    N_Sky:=DbFieldNum('SKY');
    N_SD_S:=DbFieldNum('SD_S');
    N_St_P:=DbFieldNum('ST_p');
    N_Mx_z:=DbFieldNum('MX_Z');
   end;
  rec[N_Name]:=Paramstr(1);
  DbWrNum(N_MJD,MJD);
  DbWrNum(N_NS, NS);
  if CSN>0 then
   begin
    DbWrNum(N_Row,Cat_Im[CSN].row);
    DbWrNum(N_Col,Cat_Im[CSN].col)
   end
  else
   begin
    DbWrNum(N_Row,0);
    DbWrNum(N_Col,0)
   end;
  DbWrNum(N_D_Ang,0 {D_Ang:no automatic rotation yet written} );
  DbWrNum(N_Sky,Sky_Av_All);
  DbWrNum(N_SD_S,Sky_SD_All);
  DbWrNum(N_St_P,Star_Perc);
  DbWrNum(N_Mx_z,Mx_z);
  if DbWrite(0,rec) then;
  DbClose;


  write(Star_list,cl,'Histogram of the file:');
  for k:=0 to 16 do
   begin
    write(Star_list,cl,k*15:3);
    for m:=0 to 14 do write(Star_list,Histogram[15*k+m]:5)
   end;
  write(Star_list,cl,255,Histogram[255]:5);
  for m:=1 to 14 do write(Star_list,'  +',m:2);

  close(Star_list);
 {end of processing the image}


 {changing the gray scale - at first, according to the found or given
     bottom and top levels and to a logarithmic scale, then on demand}
 repeat
  {reading new bottom/top from a string}
  if aux_s<>'' then
   begin
    val(aux_s,m,Wrong_pos);
    if Pos(' ',Aux_s)>0 then
     begin
      if Pos(' ',Aux_s)>1 then
       begin
        val(copy(aux_s,1,pos(' ',Aux_s)-1),bottom,Wrong_pos);
        if Wrong_pos>0 then Stop('')
       end;
      val(copy(aux_s,pos(' ',Aux_s)+1,length(aux_s)-pos(' ',Aux_s)),top,Wrong_pos);
      if Wrong_pos>0 then Stop('')
     end
    else bottom:=m
   end;

  if Top <= bottom then Stop('');

  {making <= bottom black, >= top white}
  if bottom>0 then for cc:=1 to bottom do with Gray256_palette[cc] do G:=0;
  for cc:=top to 255 do with Gray256_palette[cc] do G:=63;

  {preventing a too long final white step which would blind the observer:}
  Mean:=(top+6*bottom) div 7;
  if top<230 then
   begin
    for cc:=top+2 to 255 do for k:=167 to 177 do Set256_Pixel(cc,k,Mean);
    for cc:=0 to top+1 do for k:=167 to 177 do Set256_Pixel(cc,k,cc)
   end
  else
   for cc:=0 to 255 do for k:=167 to 177 do Set256_Pixel(cc,k,cc);

  m:=bottom+1; k:=top-1;

  {switching between linear and logarithmic scales}
  if Linear then
   begin
    intln:=(top-bottom)/62;
    for cc:=m to k do with Gray256_palette[cc] do
     G:=1+trunc((cc-bottom)/intln)
   end
  else
   begin
    IntLn:=ln(top-bottom)/62;
    for cc:=m to k do with Gray256_palette[cc] do
     G:=1+Trunc(ln(1+cc-bottom)/IntLn)
   end;

  {reshading and ev. new round - if not given a char<32}
  Set256_Palette(Gray256_palette);
  GoToXY(26,WhereY);
  read(ch);
  if ord(ch)<32 then Stop('');
  readln(aux_s);
  if (ord(ch) in [33 .. 47]) or (ord(ch)>57) then Linear:= not Linear
  else aux_s:=ch + aux_s;
 until false;


end.
