 {~ Faintness (and distance) --> brightness and energy domain ~}

{$N+} {to enable >double< variable because of large exponents}
{$E+}   {to run on a computer without numeric coprocessor}

program Fai2Bri; {Faintness to brightness, by Jan Hollan, 1995}

const cl=#13#10;

c=299792458.0;    {m/s : from definition of 1 m}
AU=1.49597870E11; {m   : from observation and fixed Gaussian grav. const.}
pc2AU=206264.806; {1''/1 rad:  parsec to astronomical units}
tr_year=86400*365.242199; {s in 1900}
 
pc2m=AU*pc2AU {=3.0856776E16 parsec to meters};
ly2pc=c*tr_year/pc2m {=0.9460528/3.0856776 light year to parsecs};

sr1pc=pc2m*pc2m; {m2: surface of 1 sr area of sphere of 1 pc radius}

                {0 mag levels of brightnesses, i.e. flux densities}
Mbol0=3.055E28;  {W, defined at IAU Gen Assembly in Kyoto, August 1997}
j0vis=2.54E-6;  {lm.m-2 : visual photopic}
j0V=3.2E-9;     {W.m-2  : V filter}  
j0B=6.5E-9;              {B filter}
j0U=2.8E-9;              {U filter}
{j0bol: see beginning of the programme}

var faintness,dist:real; j:integer; Par:string; bri:double; pi4,j0bol:real
;

function Dn(n:byte;x:double):string;
 {semilogarithmic expression with n (4, e.g.) decimal places 
                      in the form #.dddd*10^##}
var fullexp,mantt:string; mant:real; j:integer;
begin
 str(x,fullexp);
 val(copy(fullexp,1,15),mant,j);
 str(mant:n+3:n,mantt);
 j:=pos('+',fullexp);
 if j=0 then j:=pos('-',fullexp);
 Dn:= mantt+'*10^'+copy(fullexp,j,1)+
                                +copy(fullexp,length(fullexp)-1,2)
end;

function D2(x:double):string;
  {semilogarithmic expression with 2 decimal places in the form #.dd*10^##}
begin
 D2:= Dn(2,x);
end;

begin
 pi4:=4*pi;
 j0bol:=Mbol0 / (100*sr1pc*pi4);  {bolometric, defined in Kyoto}
 {j0bol=2.55329E-8;}

if (Paramcount=0) or (Paramstr(1)>'9') or (pos('-h',Paramstr(1))>0) then
 begin
  writeln(cl,
'Fai2Bri # [#] [<unit>] converts faintness / 1 magnitude',cl,
'        to the language of brightness and energy.',cl,
'           The second parameter is distance,',cl,
'              the third one its unit',cl,
'              (default pc,',cl,
'               A or U  for astronomical unit,',cl,
'               l or s  for light year,',cl,
'               m       for meter;',cl,
'                 just its first character is read).',cl,cl,

'Example:',cl,
'         Fai2Bri -26.8 1 AU',cl,
'   will say you how much shines the Sun (with faintness of -26.8 mag,',cl,
'   -- visual one is meant, the bolometric one would be -26.86221 mag)',cl,cl,

'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'
);
  halt
 end;
val(paramstr(1),faintness,j);
bri:=exp(ln(10)*(-faintness)/2.5);
writeln;
writeln('Faintness diference of '+paramstr(1)+' mag corresponds to');
writeln('brightness ratio = ',bri:9:4, ',  radius ratio = ',sqrt(bri):7:3);
writeln('                 =',Dn(3,bri), ',            = ',Dn(3,sqrt(bri)));
writeln;
writeln(
' Faintness  = '+paramstr(1)+' mag  implies: '); 
writeln('            =',Dn(4,bri*j0bol),' W .m^-2 (bolometric)');
writeln ('  (error of conversion is up to 5 % further:)');
writeln(' brightness =',d2(bri*j0vis),' lm.m^-2 (visual)');
writeln('            =',d2(bri*j0V),' W .m^-2 (through V filter)');
writeln('            =',d2(bri*j0B),' W .m^-2 (through B filter)');
writeln('            =',d2(bri*j0U),' W .m^-2 (through U filter)');

if paramcount>1 then
begin
 val(paramstr(2),dist,j);
  if dist<=0 then begin writeln('distance <= 0 ???'); halt end;
 if paramcount>2 then
  begin
   par:=Paramstr(3);
   par:=UpCase(par[1]);
   case par[1] of
   'P':;
   'A','U': dist:=dist/pc2AU;
   'L','S': dist:=dist*ly2pc;
   'M'    : dist:=dist/pc2m;
   end
  end;
 writeln;
 writeln('Absolute faintness (in 10 pc) assuming distance of',Dn(3,dist),'pc is ');
 writeln('              ',faintness-5*ln(dist/10)/ln(10):7:4,' mag');
 bri:=bri*sqr(dist);
 bri:=bri*sr1pc;
 writeln('luminous intensity');
 writeln('             ',d2(bri*j0vis),' cd');
 writeln('luminous output (assuming the source is isotropic) woud be');
 writeln('             ',d2(bri*j0vis*4*pi),' lm');
 writeln('bolometric (electromagnetic) output (assuming the source is isotropic) woud be');
 writeln('             ',Dn(4,bri*j0bol*4*pi),' W');

{ writeln(cl+
'(Isotropic source at 10 pc with bolometric faintness 0 mag'+cl+
' has an electromagnetic output 3.055 E28 W exactly)'); }

end;

end.
