 {~ Faintesses of stars from a matrix of their comparisons, rough work ~}

program Faint;
 {Adapted by Jan Hollan from a product of (c) Skware 1989 (= from Martin Skutek)}

uses DbBridge, crt ;

const
cl=#13#10;
h1=cl+
'Faint <name of a dBase file> [<output file>]'+cl+cl+

'Selfconsistent computation of fainesses of stars for which a matrix of'+cl+
'their comparisons exists -- these differences are to be contained'+cl;
h2=
'in a properly sorted input dBase file, with no combination of stars'+cl+
'missing (missing values should be coded as 99.9). See example Faint.dbf'+cl+cl;
h3=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;

oc1='     ^    ^'                               +cl+
    '     |    residual sum of O-C differences' +cl+
    '     computed faintness'                   +cl;

oc2= cl+
'  The matrix elements have the following meaning:' +cl+cl+

'       Observed difference'           +cl+
'       (the matrix is supplemented to be antisymmetric)'+cl+cl+

'       O-C in the first approx.    O-C in the final approximation';

C=12;
eps=0.06;

 type
      Tpole = array[1..c] of real;

 var  VUd:array[1..c] of char;
      Tab:array[1..c,1..c,1..4] of real;
      Pomr,Pomsum,OSlab:Tpole;
      Pomn:array[1..c] of integer;
      z:char;
      i,j,cs:integer;
      s,a,b:real;
      Chyb:Boolean;
      Vystup:text;
      ch:char;
      NEnOfFi : Boolean;
      aux, DbFileName: string;
      NoR : longint;

Procedure Help;
 begin
  write(h1,h2,h3); halt;
 end;


Procedure RNVal(str:string; var r:real);
var J: integer;
begin
  val(str,r,J); r:=-r;
  If J>0 then Begin
   writeln ('non-numeric content in ',str,' starting at character', J);
   r:= 99.9  End
end;

 procedure CtiDif;

  var i,j:integer;

  begin
   if ParamCount>0 then DbFileName:=ParamStr(1) else help;
   if not DbInit(DbFileName) then
    begin
     writeln(cl,'     Sorry, dBase file not found.',cl,cl,'Usage:');
     help;
    end;
   NoR:=1; cs:=2;
   NEnOfFi:= DbRead(NoR,rec);   inc(NoR);
   VUd[1]:= rec[2,1]; VUd[cs]:= rec[1,1]; RNval(rec[4],Tab[1,cs,1]);
   Tab[cs,1,1]:=-Tab[1,cs,1];
   NEnOfFi:= DbRead(NoR,rec);   inc(NoR);
   while rec[2,1]=VUd[1] do
         begin
           inc(cs);
           VUd[cs]:= rec[1,1]; RNval(rec[4],Tab[1,cs,1]);
           Tab[cs,1,1]:=-Tab[1,cs,1];
           NEnOfFi:= DbRead(NoR,rec);   inc(NoR);
         end;
   dec(NoR);
   for i:=2 to cs-1 do
    for j:=i+1 to cs do
     begin
           NEnOfFi:= DbRead(NoR,rec);   inc(NoR);
           RNval(rec[4],Tab[i,j,1]);
           Tab[j,i,1]:=-Tab[i,j,1];
     end;
   DbClose;
  end;

 procedure Vypocti;

  var s:real;
      i,j:integer;

  begin
   s:=0;
   for i:=1 to cs do
    begin
     if i>1 then s:=Pomsum[i]/Pomn[i];
     for j:=1 to cs do
       if abs(Tab[i,j,1])<99 then
         begin
          Tab[i,j,2]:=Tab[i,j,1]+s;
          Pomsum[j]:=Pomsum[j]+Tab[i,j,2];
          Pomn[j]:=Pomn[j]+1;
         end;
     end;
    for i:=1 to cs do begin Pomsum[i]:=Pomsum[i]/Pomn[i];
                            OSlab[i]:=Pomsum[i] end
  end;


 procedure Zamen(var i,j:integer);

  var k:char;
      l,m:integer;
      o:real;

  begin
   k:=VUd[i+1];
   VUd[i+1]:=VUd[j];
   VUd[j]:=k;
   for m:=1 to cs do
    begin
     o:=Tab[i+1,m,1];
     Tab[i+1,m,1]:=Tab[j,m,1];
     Tab[j,m,1]:=o;
     Tab[m,i+1,1]:=Tab[m,j,1];
     Tab[m,j,1]:=-o;
    end;
  end;


 procedure Setrid;

  var I,J:integer;

  begin
   for I:=1 to cs-2 do
    begin
     J:=I;
     repeat
      J:=J+1;
     until (J=cs+1) or (abs(Tab[i,j,1])<99);
     if J<=cs then Zamen (I,J) else
      begin
       Writeln('Non-contiguous matrix, no result.');
       halt;
      end;
    end;
  end;

 procedure DoplnR(K:integer);

  var I,J:integer;

  begin
   for I:=1 to cs do
    for J:=1 to cs do
     if abs(Tab[I,J,1])<99 then
           Tab[I,J,K]:=Tab[I,J,1]-(PomSum[J]-PomSum[I]);
  end;

 procedure SumaR;

  var I,J:integer;

  begin
   for I:=1 to cs do
    begin
     PomR[I]:=0;
     for J:=1 to cs do if abs(Tab[I,J,1])<99
        then PomR[I]:=PomR[I]+Tab[I,J,3]-Tab[J,I,3];
    end;
  end;

 function Rozhodni(Pom:Tpole):Boolean;

  var I:integer;

  begin
   Rozhodni:=false;
   for I:=1 to cs do if abs(Pom[I])>Eps then Rozhodni:=true;
  end;

 procedure Optimalizuj(Z:char);

  var I,Por:integer;
      R:real;
      JM:char {string[8]};

  begin
   R:=0;
   if (Z='a') or (Z='A') then
    begin
     Por:=1;
     for I:=1 to cs do if abs(PomR[I])>abs(PomR[Por]) then Por:=I;
     R:=(PomR[Por])/Pomn[Por]/2;
    end
   else
    begin
     Writeln;
     Writeln('Hvezda   slabost   suma odchylek');
     for I:=1 to cs do
      begin
       Writeln(VUd[I],Pomsum[i]:8:2, PomR[i]:8:2);
      end;
     write('Zadej hvezdu (mezera = konec) a zmenu:');
     Readln(JM,R);
     Por:=0;
     for I:=1 to cs do if VUd[I]=JM then Por:=I;
     if Por=0 then if JM=' ' then for I:=1 to cs do PomR[i]:=0;
    end;
   PomSum[Por]:=PomSum[Por]-R;
   PomR[Por]:=PomR[Por]-2*R*Pomn[Por];
   for I:=1 to cs do
    if (I<>Por) and (abs(Tab[I,Por,1])<99)
       then PomR[I]:=PomR[I]+R;
  end;



 Procedure Vysl;

  var I,J,K:integer;

  begin
   S:=0;
   K:=0;
   for I:=1 to cs do
    begin
     for J:=1 to cs do if abs(Tab[I,J,1])<99 then S:=S+sqr(Tab[I,J,4]);
     K:=K+Pomn[I];
    end;
   S:=sqrt(S/(K-1));
  end;

  procedure TiskVysl;

   var I:integer;

   begin
      write(Vystup,'number of diff.');
      for i:=1 to cs do write(Vystup,Pomn[i]:4,'   '); writeln(Vystup);
            {Number of non-empty elements}
      write(Vystup,'design. of star');
      for i:=1 to cs do write(Vystup,'   ',VUd[i],'   '); writeln(Vystup);
            {Name of Star}
      write(Vystup,'first approx.  ');
      for i:=1 to cs do write(Vystup,10*OSlab[i]:4:0,'   '); writeln(Vystup);
            {First approximation of its observed faintness}
      writeln(vystup);  
      for i:=1 to cs do
       begin
         write(Vystup,VUd[I],10*PomSum[I]:7:1,10*PomR[I]:5:1,'  ');
            {Name, Final approximation, residual sum of O-C}
         for j:=1 to cs do if(abs(Tab[i,j,1])<99) then
               write(Vystup,10*Tab[i,j,1]:4:0,'   ')
            {source data}
               else write(Vystup,' ....  '); writeln(Vystup);
         write(Vystup,'               ');
         for j:=1 to cs do if(abs(Tab[i,j,1])<99) then
               write(Vystup,10*Tab[i,j,3]:3:0,10*Tab[i,j,4]:3:0,' ')
            {O-C of the first approximation and of the final approximation}
               else write(Vystup,' ....  '); writeln(Vystup);
       end;
       writeln(vystup,oc1);
       Writeln(Vystup,'Resulting mean standard deviation:',S:4:2,' AS');
       writeln(vystup,oc2);

      write('             ');
      for i:=1 to cs do write(Pomn[i]:4,'   '); writeln;
      write('             ');
      for i:=1 to cs do write('  ',VUd[i],'    '); writeln;
      write('             ');
      for i:=1 to cs do write(10*OSlab[i]:4:0,'   '); writeln;
      for i:=1 to cs do
       begin
         write(VUd[I],10*PomSum[I]:7:1,10*PomR[I]:5:1);
         for j:=1 to cs do if(abs(Tab[i,j,1])<99) then
               begin HighVideo; write(10*Tab[i,j,1]:4:0,'   '); LowVideo end
               else write(' ....  '); writeln;
         write('             ');
         for j:=1 to cs do if(abs(Tab[i,j,1])<99) then
               write(10*Tab[i,j,3]:3:0,10*Tab[i,j,4]:3:0,' ')
               else write(' ....  '); writeln;
       end;
     Writeln('Mean standard deviation:',S:4:2,' AS');
   end;


 begin
  if Paramcount>0 then
   if (pos('?',paramstr(1))>0) or (pos('-h',paramstr(1))>0) then help;
  Chyb:=false;
  for I:=1 to 12 do
   begin
    PomSum[I]:=0;
    Pomn[I]:=0;
    for j:=1 to 12 do
     for cs:=1 to 4 do
     Tab[I,j,cs]:=0;
    Tab[I,I,1]:=99.9;
   end;
  CtiDif; {there the input file, whose name is the first parameter, is
           opened and read into the working arrays}
  Setrid;
  Vypocti;
  DoplnR(3);
  SumaR;
  Z:='A';
  while Rozhodni(PomR) do Optimalizuj(Z);
  DoplnR(4);
  Vysl;
  if ParamCount>1 then aux:= ParamStr(2) else aux:= 'Faint.txt';
{  if not rt_rewrite(vystup,aux) then halt;}
  assign(vystup,aux);
  rewrite(vystup);
  TiskVysl;
  close(Vystup);
 end.
