 {~ Heliocentric correction (a quantity, meaningful for variable stars) ~}

program Hel_Cor;  {by Jan Hollan}

uses {J.H's units:} solar_Ut,planets,str_num,params,un_str_m;

var j:byte;
const Date_given:boolean=false;

procedure Help;
 begin
  writeln(

'Hel_Cor   RA# De# dDD[.MM[.YY]] t{ # | HH[:MM[:SS]] } UT#',cl2,

' gives a so-called heliocentric correction for a fixed star,',cl,
' with the given J2000 Right Ascension and Declination.',cl2,

'The following parametes are optional:',cl,
' D  date (initial value is the current one)',cl,
' T  time (default is the current one), in hours or as hour:minute:second',cl,
' UT is the t-UTC difference / 1 h (default 1 from Nov. to March, else 2)',cl2,

'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl
);

  halt;
 end;

begin
if paramcount=0 then help
else for j:= 1 to paramcount do
 begin
  Par:=paramstr(j);
  Par:=__cvtstr(Par,_To_UpCase_Str);
  if (Par[1]='/') or (Par[1]='-') then
    Par:=copy(Par,2,length(par)-1);
  case Par[1] of
   '?','-': Help; {- added to respect unix --help even without the help}
   'R' : if not Par_RA then help;
   'D' : if Par[2]='E' then
          if not Par_Decl then help
         else
          begin
           if not Par_date then help;
           Get_JD;
           Date_given:=true;
          end;
   'T':  if not Par_Time then help;
   'U':  if not Par_UT then help;
  end;
 end;
if Date_given then
 writeln('For Julian date = ',JD:11:3)
else
 writeln('For then present moment (Julian date = ',JD:11:3,')');
write('and the fixed star with the given J2000 RA = ',RA:6:2,', Decl. = ',Decl:6:2);
writeln('is the heliocentric correction ',
          SRe(5,1,Heliocentric_correction_s(JD,RA,Decl)));
end.
