 {~ Length of deformed main circles in "true-polar-distance" maps ~}

program Map_Wires;  {by Jan Hollan}

uses angles_o, params,str_num;

const
h1=
'Map_wires # R# F# T# S#'+cl2+

'   helps to make a grid for a rotating star chart, equidistant in declination.'+cl2+

'Its parameters are:'+cl2;
h2=
' # is the latitude (default 50) or its lower limit (same as F#)'+cl+
'R    distance "pole-equator" (default 100)'+cl+
'F    lower limit of interval of latitudes'+cl+
'T    upper limit      (default equal to lower limit)'+cl;
h3=
'S    step of latitude (default 1) - meaningful if T-F>=S'+cl2;
h4=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;


e1=cl+
'The table helps to make a basic grid for rotatable star charts.'+cl+
'The grid consists of a wire symbolizing the horizon and a wire going'+cl+
'from east through zenith to west.'+cl;
e2=
'   For its construction, apart from having a pair of pliers and a basic'+cl+
'knowledge of euclidean geometry, knowing the lengths of some wire segments'+cl+
'is vital for getting an accurate result.'+cl;
e3=
'   The table gives for a given latitude the lengths of the northern half of'+cl+
'the horizon, the southern part, their sum (whole horizon), and a length of'+cl+
'a semi-circle going from east through zenith to west.'+cl;
e4=
'   The values hold for a pole-centered azimuthal map with true declination,'+cl+
'(The map is south-pole centered for negative latitudes, and then the meaning'+cl+
'of zenith/nadir is reversed.) The distance from pole to equator was chosen as'+cl+
'Radius_90 = ';


 From:real=50;  {90 - (starting angle between central pole
                            and pole of the main circle) / 1 degree ;
                for horizon length on rotating charts it is the latitude }
 Up_to:real=-90; {90 - final angle; this value ensures computing
                                    just From as a default}
 Step:real=1;
 Radius_90:real=100;

var
 Fi_d, {actual latitude in degrees}
 Fi_r,                { in radians instead of degrees}
 angle,cotg_Fi,length_incr, length_sum, length_long: real;
 j,wrong_char: integer;

 procedure help;
  begin
   write(h1,h2,h3,h4);
   halt;
  end;

 Function length_el(x:real):real;  {length of a horizon element}
  var cFsx:real;
  begin
   cFsx:=cotg_Fi*sin(x);
   length_el:= sqrt( sqr(pi/2-arctan(cFsx))
                   + sqr(cotg_Fi*cos(x)/(1+sqr(cFsx))) )
  end;


 PROCEDURE qgaus(a,b: real; VAR ss: real);
                                         {taken from the RECIPES package}
  (* Programs using routine QGAUS must externally define
   a function func(x:real):real which is to be integrated *)
                  {: I call it length_el}
  VAR
   j: integer;
   xr,xm,dx: real;
   w,x: ARRAY[1..5] OF real;
  BEGIN
   x[1] := 0.1488743389;
   x[2] := 0.4333953941;
   x[3] := 0.6794095682;
   x[4] := 0.8650633666;
   x[5] := 0.97390652;
   w[1] := 0.2955242247;
   w[2] := 0.2692667193;
   w[3] := 0.2190863625;
   w[4] := 0.1494513491;
   w[5] := 0.06667134;
   xm := 0.5*(b+a);
   xr := 0.5*(b-a);
   ss := 0;
   FOR j := 1 to 5 DO BEGIN
      dx := xr*x[j];
      ss := ss+w[j]*(length_el(xm+dx)+length_el(xm-dx))
   END;
   ss := xr*ss
  END;

 procedure one_Fi;
  var j: integer;
  begin
   if abs(Fi_d)>=1 then
    begin
     Length_sum:=0;
     Fi_r:=Fi_d * pi / 180;
     cotg_Fi:= cos(Fi_r)/sin(Fi_r);

     {Further, integration goes in 3-degrees increments.
               Angle is measured from intercept to short part.}

     for j:= -30 to -1 do     {long half}
     begin
      qgaus(pi*j/60.0,pi*(j+1)/60.0,length_incr);
      Length_sum:=Length_sum+length_incr
     end;
     Length_long:=Length_sum*4*radius_90/pi;

     for j:= 0 to 29 do       {shor half}
     begin
      qgaus(pi*j/60.0,pi*(j+1)/60.0,Length_incr);
      Length_sum:=Length_sum+Length_incr
     end;
     Length_sum:=Length_sum*4*radius_90/pi;
     write(Fi_d:3:0,
      (Length_sum-Length_long):7:1,Length_long:7:1,Length_sum:7:1);
    end
   else
    write(Fi_d:3:0,
     Radius_90*2:7:1,radius_90*2*(1+pi):7:1,radius_90*2*(2+pi):7:1);


   Fi_r:=pi/2-Fi_r;
   if (Fi_r>0.02) and (Fi_r<pi-0.02) then   {length of circle segment
                         between the intercepts through zenith}
    begin
     cotg_Fi:= cos(Fi_r)/sin(Fi_r);
     length_sum:=0;
     for j:= 0 to 29 do
     begin
      qgaus(pi*j/60.0,pi*(j+1)/60.0,Length_incr);
      Length_sum:=Length_sum+Length_incr
     end;
     Length_sum:=Length_sum*4*radius_90/pi;
     writeln(Length_sum:7:1);
    end
   else
     if Fi_r<pi/2 then
      writeln(Radius_90*2:7:1)
     else
      writeln(radius_90*2*(1+pi):7:1);

  end;

begin
if paramcount>0 then
 for j:=1 to paramcount do
  begin
   Par:=paramstr(j);
   if Par[1] = '/' then Par:=copy(Par,2,length(Par)-1);
   val(Par,angle,wrong_char);
   if wrong_char=0 then
     From:=angle
   else
    case UpCase(Par[1]) of
     'R': Radius_90:=par_num(Par);
     'F': From:=par_num(Par);
     'T': Up_to:=par_num(Par);
     'S': Step:=par_num(Par);
     '?': help;
    end;
  end;
if Up_to<From then Up_to:=From;

Fi_d:=From;
writeln(e1,e2,e3,e4,Radius_90:5:1,cl,cl,
 'Lat. north  south  whole  zenith (nadir for southern latitudes)');
repeat
 one_Fi;
 Fi_d:=Fi_d+Step;
until Fi_d>Up_to;

end.

