 {~ Lunar ephemeris ( daily  |  main phases only |  eclipses only ) ~}

Program moon; {by Jan Hollan, using a unit co-written by D.Motl}
(*  Copyright (C) 1999 Jan Hollan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
    or get http://www.gnu.org/copyleft/gpl.html


*)

uses angles_o, solar_ut, str_num, Un_Str_M {instead of Unit_Str by Blaise Computing, Inc.},
     planets {a unit co-written by David Motl},
     params,
     precesse ;
const

{ A queer expressing of times of main phases (over 24 hours, up to 48)
  happens in those occasions when computing daily ephemeris,
  when the starting time was later in the day than the moment of that phase.
  For /t0, you will get no times over 24,
  for /t23.59, all of them  will be over 24.
 }

cl=#13#10;
h1=cl+
'A program from Jan Hollan, N. Copernicus Obs. and Planetarium in Brno, 1999,'+cl+
' (subject to the GNU General Public License, http://www.gnu.org/copyleft,'+cl+
'  source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;
h1a=
'gives (geocentric) lunar ephemeris:'+cl+cl+
'Moon dDD[.MM[.YY]] tHH[.MM[.SS]] UT# TO L<longitude> F<latitude>'+cl+
'     { N# { M | E | C# } | C } F{G|J} TT#[unit] '+cl2;
h2=
'D and T are initially taken from the system date and time'+cl+
'UT is the t-UTC difference / 1 h (default 1 from Nov. to March, else 2)'+cl+
'N  number of next lines to be shown (default 3)'+cl+
'M  just the four main phases of the Moon should be shown'+cl;
h3=
'E  just eclipses should be shown'+cl+
'Y  year should be given on each line'+cl2+

'TO give topocentric position (instead of a geocentric one)'+cl+
'L  longitude / 1 degree (default that of '
  +MyLocation_Name+', '+MyLocation_longitude_string+')'+cl;
h4=
'F  latitude  / 1 degree (default that of '
  +MyLocation_Name+', '
 +MyLocation_latitude_string+')'+cl+
'F{G|J} forces Gregorian (Julian) dating even outside the standard interval'+cl;
h5=
'C  give just coordinates (as an input for further program, probably)'+cl+
'C# compute the moments of the given colongitudo'+cl2+

'(no parameters are needed - then the next 3 days are shown as well)'+cl;

 Next:word = 3;
 All_days :boolean = true;   {each following line is one day later}
 Eclipse: boolean = false;   {write just eclipses}
 Lunar:boolean=false;        {Lunar eclipse was encountered}
 Lun_s:boolean=false;        {Lunar semi-shadow eclipse was encountered}
 RepeatYear:boolean=false;

 moon_rad:real=0.27248; {in Earth radii}

 just_coord:boolean=false;
 Topocentric:boolean=false;
 Colongitudo_search:boolean=false;
 Phase_int:real=0.2; {search for main phases is done in +- Phase_int}
 lang:byte=1; {English}

var
 Name,phase:string;
 l,b,r,lg,bg,{rg,}f,ls, lg_old, phase_time,ab,sol_arad,moon_arad : real;
 rs,sol_rad,T_orig:real;
 Colongitudo_wanted,colongitudo_rel,colongitudo_rel_old,Colong_time,
 ra_ENP, decl_ENP,earth_long, Phase_int_i,el :real;
 AUiER:real;
 i: byte;
 last_phase :shortint;
{ code , code_aux : integer;}
 N_lines,old_YYYY :word;
 j : integer;

procedure Help;
begin
 write(h1,h1a,h2,h3,h4,hTT,h5); halt
end;

Procedure Moon_topo(JD_n:real);
begin
   LB_RD(l,b,RA,Decl);
   SM:=Get_SM(JD_n,Lambda);
   A:=deg_to_rad(360*SM/24-RA); h:=deg_to_rad(Decl);
   ah_to_td(pi/2-fi,A,h);
   r:=r-sin(h);
   h:=h - arctan(cos(h)/r);
   lg:=A; bg:=h;
   A:=rad_to_deg(A);
   h:=rad_to_deg(h);
   ah_to_td(-(pi/2-fi),lg,bg);
   lg:=deg_to_rad(360*sm/24)-lg;
   ra_d2l_b(epsi_rad,lg,bg);
   l:=rad_to_deg(lg);
   b:=rad_to_deg(bg);
end;

procedure Moon_phase(JD_n:real);
var jd_tt:real;
 {jd for Temps Terrestric, a bit greater than jd_n most times}
begin
  if tt_ut_set then
   jd_tt:=jd_n+tt_ut
  else
   jd_tt:=jd_n+tt_utc_in_days(jd_n);
  Planet_hel(3,jd_tt{jd_n originally},l,b,rs,Name);
  ls:=bas_int(l+180,360);
  Moon_geo(jd_tt{jd_n originally},l,b,r,f);
  if Topocentric then Moon_topo(JD_n);
  lg:=Bas_Int(l-ls,360);
  ang_dist(ls,0,l,b,el);
  Name:=DoW(JD_n,t_UT,lang);
  if colongitudo_search then
   begin
    Epsi:=Epsilon(JD_n);
    if not PrecInit(2000+(JD_n-JD2000)/JulCentury,2000) then halt;
    ra_ENP:=0; decl_ENP:=90;
    Precess(ra_ENP,decl_ENP);
    LB_RD(l,b,RA,Decl);
    precess(ra,decl); {to J2000.0}
    earth_long:=subterr_longD(11,1,jd_n,ra,decl,0);
    if earth_long>180 then earth_long:=earth_long-360;
    colongitudo_rel:=Bas_Int(l-ls+rad_to_deg(r*sin(deg_to_rad(l-ls))/(rs*AUiER))
                         -90-earth_long-colongitudo_wanted,360);
  end;
end;


procedure Wr_Basic_data;
begin
 write(l:6:2,'  ',b:6:2,' = '+deg_to_dm(b),'  ',
       r:5:1,lg/360*29.53059:8:1);
 if lg>180 then lg:=lg-360;
 write(lg:6:0,'  ',Name,' ',mm:2,'-',nd:2);
 inc(N_lines);
end;


begin
  AUiER:=149600/6.378; {Astronomical unit in Earth radii}
  sol_rad:=0.695550/149.504; {in AU}

  if paramcount>0 then for j:= 1 to paramcount do
   begin
    Par:=paramstr(j);
    Par:=__cvtstr(Par,_To_UpCase_Str);
    if (Par[1]='/') or (Par[1]='-') then
      Par:=copy(Par,2,length(par)-1);
    case Par[1] of
      '?','-': Help; {- added to respect unix --help even without the help}
      'C': if length(Par)=1 then just_coord:=true
           else
            if Par[2]='Z' then lang:=2
            else
             begin
              colongitudo_search:=true;
              colongitudo_wanted:=par2r(Par);
              All_days:=false;
             end;
      'D': if Par[2]<':' then if not Par_Date then halt;
      'E': if length(Par)=1 then
            begin
             Eclipse:=true;
             All_days:=false
            end;
      'F': if length(par)>1 then
            case Par[2] of
              'J': ForcedJulian:=true;
              'G': ForcedGregorian:=true;
             else
              if not Par_Fi then halt;
            end;
      'L': if length(Par)>1 then if not (Par[2]='S') then
             if not Par_Lambda then halt;
      'M': if length(Par)=1 then All_days:=false;
      'N': if length(Par)>1  then Next:=Par2I(Par);
      'T': if Par[2]='O' then Topocentric:=true
           else if Par[2] in ['I','Y'] then   {Tycho input for map_bsc}
                else if not Par_Time_or_TT then halt;
      'Y': RepeatYear:=true;
      'U': if Par[2]='T' then if not Par_UT then halt;

    end; {of case}
   end; {of for - of processing parameters}

  Get_JD;
  if not tt_ut_set then tt_ut:=tt_UTC_in_days(JD);
  if not t_UTC_given then
   if MM in [4..10] then t_UT:=2 else t_UT:=1;
  if Just_coord then
   begin
    Moon_phase(JD);
    LB_RD(l,b,lg,bg);
    writeln('RA',lg:4:3,' De',bg:4:3);
    halt(1);
   end;

  if All_days then
   writeln(cl,
'Moon ephemeris (elongation from the Sun in longitude only, western as negative)')
  else
   if eclipse then
    writeln('Ephemeris of lunisolar eclipses')
   else
    if colongitudo_search then
     writeln(
 'Moments of the selenographic colongitudo of the Sun =',colongitudo_wanted:6:1,
   ' degrees')
    else
     writeln('Moon ephemeris - the four main phases only');
  if Topocentric then
    writeln(cl,
    'The positions are topocentric, as they would be with no refraction from',cl,
    ' terrestrial longitude', rad_to_deg(lambda):6:1,' degrees and latitude',
     rad_to_deg(fi):6:1,' degrees');

  writeln;
  writeln('The starting moment is, expressed in the UTC:');
  writeln('           ',yyyy,'-',mm:2,'-',nd:2,'  ',h_min_s(UT));
  writeln('And in the USED TIME, which IS ',t_UT:4:1,' h ahead of th UTC:'+cl+
   DoW(JD,t_UT,lang),'  ',yyyy,'-',mm:2,'-',nd:2,'  ',h_min_s(Bas_Int(UT+t_UT,24)));
  if tt_UT>0.001 then
   if tt_ut_set then
    writeln('Terrestric Time - UT assumed : ',tt_UT*24:6:2,' h',cl)
   else
    writeln('Terrestric Time - UT assumed initially : ',tt_UT*24:6:2,' h',cl)
  else
   writeln;

  WRITELN('Julian date  : ',JD:13:4);
  writeln;
 if All_days then
  writeln(
'(each following line is incremented by 1 day, outside eclipses lines)',#13#10)
 else if not Colongitudo_search then
  writeln('(each line is computed for the corresponding difference of',#13#10,
    'ecliptical longitude of Sun and Moon - multiple of 90 degrees)',#13#10);

 if all_days then   {normal heading}
  begin
   write(
'longitude  latitude     distance  old   elongation      date  time phase');
   if Topocentric then writeln(' height') else writeln;
   write(
'degrees  degrees  dd:mm   E.rad.  days  degrees         mm-dd hh:mm 2 =');
   if Topocentric then writeln(' degrees') else writeln;
    writeln(
'                                                                   full');
  end
 else
  if eclipse then   {eclipse heading}
   begin
    write(
'longitude latitude distance          UTC + ',t_UT:4:1,' h');
    if Topocentric then writeln('                    height')
    else writeln;
    write(
'degrees   degrees  E.radii');
    if Topocentric then
     writeln('                                          degrees')
    else writeln;
    writeln
   end
  else
   if Colongitudo_search then
    begin
     writeln(
'Lunar ecliptical coordinates                 The moment');
     write(
'                                           (UTC + ',t_UT:4:1,' h)');
     if Topocentric then writeln(
                                 '     angular') else writeln;
     write(
'longitude   latitude    distance');
     if Topocentric then writeln(
             '                              height') else writeln;
     write(
'degrees  degrees  dd:mm   E.rad.          yyyy-mm-dd hh:mm');
     if Topocentric then writeln('    degrees') else writeln;
     writeln
    end
   else         {main phases heading}
    begin
     write(
'longitude  latitude     distance               date  time   phase');
     if Topocentric then writeln('       height') else writeln;
     write(
'degrees  degrees  dd:mm   E.rad.          yyyy-mm-dd hh:mm  2=full');
     if Topocentric then writeln('     degrees') else writeln;
     writeln
    end;

Moon_phase(JD);
old_YYYY:=YYYY-1;
if colongitudo_search then
 begin
  if (360-colongitudo_rel)>12 then
   begin
    JD:=JD+(trunc(360-colongitudo_rel) div 12);
    Moon_phase(JD);
    colongitudo_rel_old:=colongitudo_rel
   end
 end
else
 last_phase:= trunc(lg/90) mod 4;
N_lines:=0;

while N_lines <= Next do
begin
 Moon_phase(JD);
 if Colongitudo_search then
  begin
   if Colongitudo_rel<13 then
    begin
     Colong_time:=1-Colongitudo_rel/(Colongitudo_rel+360-colongitudo_rel_old);
     Moon_phase(JD-1+Colong_time-Phase_int);
     colongitudo_rel_old:=colongitudo_rel;
     Moon_phase(JD-1+Colong_time+Phase_int);
     Colong_time:=Colong_time-Phase_int+2*Phase_int*
                 (1-(Colongitudo_rel)/(Colongitudo_rel+360-colongitudo_rel_old));
          Moon_phase(JD-1+Colong_time-Phase_int/10);
     colongitudo_rel_old:=colongitudo_rel;
     Moon_phase(JD-1+Colong_time+Phase_int/10);
     Colong_time:=Colong_time-Phase_int/10+0.2*Phase_int*
                 (1-(Colongitudo_rel)/(Colongitudo_rel+360-colongitudo_rel_old));
     JD:=JD-1+colong_time;
     Moon_phase(JD);
     write(l:6:2,'  ',b:6:2,' = '+deg_to_dm(b),'  ',
        r:5:1);
     if RepeatYear or (YYYY>old_YYYY) then
      begin
       write('  ',Name,' ',yyyy:4,'-',mm:2,'-',nd:2);
       old_YYYY:=YYYY
      end
     else
      write('  ',Name,'      ',mm:2,'-',nd:2);
     write(deg_to_dm(Bas_int(UT+t_UT,24)));
     if Topocentric then writeln(h:10:2) else writeln;
     inc(N_lines);
     JD:=JD+27;
     Moon_phase(JD);
    end;
   colongitudo_rel_old:=colongitudo_rel;
  end
 else
  if (trunc(lg/90) > last_phase)
  or ((last_phase=3) and (trunc(lg/90)=0))
  then
   begin
    T_orig:=Bas_int(UT+t_UT,24);
    if T_orig>23.9999 then T_orig:=0;
    inc(last_phase);
    last_phase:=last_phase mod 4;
    str(last_phase:3,phase);
    if last_phase=0 then phase:='   ';
    if (not Eclipse) or (last_phase in [0,2]) then
            {esp. for Topocentric case and New moon, minimum
             angular distance should be sought for, and not just zero
             difference of longitudes}
     begin
      Phase_time:=Bas_int(last_phase*90-lg_old,360)/Bas_int(lg-lg_old,360);
        {to avoid one-day non-uniformity of changes of elongation lg,
         computing the corresponding phase moment in a shorter interval}
      Phase_int_i:=Phase_int;
      for i:=1 to 4 do
       begin
         Moon_phase(JD-1+Phase_time-Phase_int_i);
         lg_old:=lg;
         Moon_phase(JD-1+Phase_time+Phase_int_i);
         lg:=Bas_int(lg-lg_old,360);
         lg_old:=Bas_int(last_phase*90-lg_old,360);
         if lg_old>10 then
          begin
           writeln(cl,'Wrong iteration of main phase moment',lg:7:2,lg_old:7:2,last_phase:3);
           lg_old:=lg_old-360;
          end;
         Phase_time:=Phase_time - Phase_int_i + Phase_int_i*2*lg_old/lg;
         Phase_int_i:=Phase_int_i/3;
       end;
      Moon_phase(JD-1+Phase_time);
      if All_days then
       begin
        write(deg_to_dm(24*Phase_time+T_orig),phase,' ');
        if Topocentric then
         begin
          Moon_phase(JD-1);
          writeln(h:6:2)
         end
        else writeln;
       end;
      Moon_phase(JD-1+Phase_time);
      if not Eclipse and not All_days then
       begin
        write(l:6:2,'  ',b:6:2,' = '+deg_to_dm(b),'  ',
           r:5:1);
        if RepeatYear or (YYYY>old_YYYY) then
         begin
          write('  ',Name,' ',yyyy:4,'-',mm:2,'-',nd:2);
          old_YYYY:=YYYY
         end
        else
         write('  ',Name,'      ',mm:2,'-',nd:2);
        write(deg_to_dm(Bas_int(24*Phase_time+T_orig,24)),' ');
        inc(N_lines);
       end;
      if last_phase in [0,2] then
       begin
        if Topocentric and (last_phase=2) then
               {to exclude parallax, which is the same for the Earth shadow}
         begin
           Topocentric:=false;
           Moon_phase(JD-1+Phase_time-Phase_int);
           lg_old:=lg;
           Moon_phase(JD-1+Phase_time+Phase_int);
           Phase_time:=Phase_time-Phase_int
               +Phase_int*2*Bas_int(last_phase*90-lg_old,360)/Bas_int(lg-lg_old,360);
           Moon_phase(JD-1+Phase_time);   {correct time of maximum depth}
           Topocentric:=true;
         end;
        ab:=deg_to_rad(abs(b));
        sol_arad:=sol_rad/rs;
        moon_arad:=moon_rad/r;
        if ( ((not Topocentric) or (last_phase=2))
             and((1+moon_rad)/r+sol_arad > ab) )
        or (Topocentric and (last_phase=0)
              and (moon_arad+sol_arad > ab))   then {eclipse}
         begin
          if Eclipse then
           begin
            write(l:6:2,'  ',b:6:2,'    ',
             r:5:1,'    ');
            if RepeatYear or (YYYY>old_YYYY) then
             begin
              write('  ',Name,' ',yyyy:4,'-',mm:2,'-',nd:2);
              old_YYYY:=YYYY
             end
            else
             write('  ',Name,'      ',mm:2,'-',nd:2);
            write(deg_to_dm(Bas_int(24*Phase_time+T_orig,24)),' ');
            inc(N_lines);
           end
          else
           if All_days then
            begin
             write(l:6:2,'  ',b:6:2,' = '+deg_to_dm(b),'  ',
                 r:5:1,' ');
             if not Topocentric then write('  ');
            end;
          case last_phase of
           0:if not Topocentric then
              begin
               if 1/r > ab then
                if moon_arad>sol_arad then
                 write('Solar full ')
                else
                 write('Solar ring ')
               else
                write('Solar part.');
              end
             else
              begin
               if Topocentric then
                begin
                 if h>-15*cos_Fi then  write('!') else write(' ');
                 if h> 0         then  write('!') else write(' ');
                end;
               if ab < moon_arad-sol_arad then
                 write('Solar full ')
               else
                if ab < sol_arad-moon_arad then
                 write('Solar ring ')
                else
                 write('Solar ',(sol_arad+moon_arad-ab)/(2*sol_arad):4:2,' ');
              end;
           2:if 1/r-sol_arad+moon_arad > ab then
              begin     {Moon touches full shadow:}
               if Topocentric then
                begin
                 if h>-10*cos_Fi then  write('!') else write(' ');
                 if h> 5         then  write('!') else write(' ');
                end;
               write('Lunar ', (1/r-sol_arad+moon_arad-ab)/(2*moon_arad):4:2,' ');
               Lunar:=true;
              end
             else
              begin     { touches just semishadow:}
               if Topocentric then
                begin
                 if h> 5   then  write(' !') else write('  ');
                end;
               write('Lun.s. ',(1/r+sol_arad+moon_arad-ab)/(2*moon_arad):4:2);
               Lun_s:=true;
              end;
           end;
          if All_Days then
           begin
            write('  ',Name,' ',mm:2,'-',nd:2);
            write(deg_to_dm(Bas_int(24*Phase_time+T_orig,24)));
            if Topocentric then
             begin
              if (last_phase=2) then   {to get back parallax of eclipsed Moon:}
                Moon_phase(JD-1+Phase_time);
              writeln(h:10:2);
             end
            else writeln;
           end
          else if Topocentric then writeln(h:7:2) else writeln;
         end
        else   {no eclipse nor occultation}
         begin
          if Topocentric and (last_phase=2) then
           Moon_phase(JD-1+Phase_time);   {to get back parallax}
          if not all_days and not Eclipse then
           begin
            write(phase);
            if Topocentric then writeln(h:17:2) else writeln;
           end
         end
       end
      else if not all_days and not Eclipse then
        begin
         write(phase);
         if Topocentric then writeln(h:17:2) else writeln;
        end;
      if not All_days then
       if Eclipse then
        JD:=JD+11
       else
        JD:=JD+5;
      Moon_phase(JD)
     end;
    if All_days then writeln;  {to get an empty line after a main phase}
   end
  else if All_days then
   if Topocentric and (N_Lines>0 )then
    begin
     Moon_phase(JD-1);
     writeln(h:16:2);
     Moon_phase(JD);
    end
   else writeln;
 if All_days then
   Wr_Basic_data;
 JD:=JD+1;
 lg_old:=lg;
end;
if All_days then if Topocentric then
 begin
  Moon_phase(JD-1);
  writeln(h:16:2)
 end
else writeln;

if Lunar then
 writeln(cl,
'"Lunar" means an eclipse, when the Moon gets into the full shadow of the Earth.',cl,
' The following number says, how deep gets the inner rim into the shadow,',cl,
' relative to the lunar angular diameter.');
if Lun_s then
 writeln(
'"Lun.s." means an eclipse, when the Moon does not touch the full shadow.',cl,
' The following number says, how deep gets the inner rim into the semi-shadow,',cl,
' relative to the lunar angular diameter. The eclipse is difficult to notice',cl,
' when the number is less then 1.');

writeln;
end.
