 {~ Time of reaching an angular height (for fixed stars, planets, Sun, Moon) ~}

Program Star_Time; {by Jan Hollan; may use B5_int.dbf}
(*  Copyright (C) 1999 Jan Hollan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

uses DbBridge, angles_o, Un_Str_M, str_num, solar_ut, params, precesse, planets,
 bsc_dir;      {just the loc_bsc constants}

const
h1=cl+
'Star_tim <point> h[u]# dDD[.MM[.YY]] UT# L# F# E#'+cl2+

'gives time and azimuth of the point being at the given angular height'+cl2;

h2=
'The point can be given through its HR number (in the Bright Star Catalogue)'+cl+
' as HR#,'+cl+
'by its Greek/Flamsteed code preceded by N as nAlp_Cas, nAlp1Sco, n42_Eri,'+cl;
h21=
'or by giving RA# (in degrees) or RAhh:mm:ss and'+cl+
'             De#   -  "  -    or DEdd:mm:"" .'+cl+
'or as C#, where # is 0 for Sun, 1 for Mercury, ... 10 for Moon'+cl2;
h22=
' H  is the wanted angular height / 1 degree (default 0)'+cl+
' HU is an "unrefracted" angular height (as if no atmosphere would be present)'+cl;
h3=
' D  is date (initial value is the current one)'+cl+
' UT is the t-UTC difference / 1 h (default 1 from Nov. to March, else 2)'+cl+
' L  is the Longitude of the observing place, default that of '
 +MyLocation_Name+','+cl;
h4=
' F  is the latitude      -               "                -  '+cl+
' E   equinox year (default 2000.0, i.e. J2000), for computing precession '+cl;

h6=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;

BSC:boolean=false;
found:boolean=false;
de_refract:boolean=true;
part_given:boolean=false;
Greek_given:boolean=false;
PrecDate:real=2000.0;
Planet_No:shortint=-1;
t_ut_sig:char='+';

var
h_unrefracted,h_geoc,t,T1,T2,l,b,r,lg,bg,rg:real;
ParVal,whystr,Name:string;
hr,hrhh,hrmm,hrss,tt: word;
hrdd,nflam:integer;
na,ndec,ParN,nname:byte;
sname,cname:string[3]; pname,switch:char;
code,j:integer;

procedure Help(s:string);
begin
 if BSC then DbClose;
 if s<>'' then writeln(cl+s+cl);
 write(h1,h2,h21,h22,h0enter);
 cont_or_halt;
 write(h0clear,h3,h4,cl,h_refraction,cl,h_dir_bsc,cl,h6);
 halt;
end;

procedure Planet_pos;
 begin
  case Planet_No of
  1,2,4..9: begin
         Planet_hel(Planet_No,jd+tt_ut,l,b,r,Name);
         Hel_to_Geo(jd+tt_ut,l,b,r,lg,bg,rg);
        end;
     0: begin
         Planet_hel(3,jd+tt_ut,l,b,r,Name);
         lg:=Bas_Int(l+180,360); bg:=0;
        end;
    10: Moon_geo(jd+tt_ut,lg,bg,rg,l{phase, not needed});
  else halt end;
  LB_RD(lg,bg,ra,decl);
 end;

procedure refine_pos(var Tref:real; sig_T:real);
 begin
  Name:=hours2hhmmss(Tref,hh,mi,ss,hs); {just to set the Babylonian time
                                           needed for Get_JD}
  Get_JD;                               {setting the Julian Date}
  Planet_pos;
  if not h2t(h_geoc,Decl,A,T,whystr) then
   begin
    writeln('The point is ',whystr,' than the given angular height');
    halt;
   end;
  RA:=RA/15;
  l:=Bas_Int(RA+sig_T*T-SM,24);
  if l>12 then l:=l-24; {the correction to a bit another RA should be small}
  Tref:=Tref+l/Sidereal2SolarTime;
 end;

begin
h:=0;
if paramcount=0 then help('')
else
 for ParN:= 1 to paramcount do
  begin
   Par:=ParamStr(ParN);
   if Par[1] in ['/','-'] then Par:=copy(Par,2,length(par)-1);
   if length(par)>1 then ParVal:=copy(par,2,length(par)-1);
   Par:=__cvtstr(Par,_To_UpCase_Str);
   switch:=Par[1];
   if switch='H' then
    if length(par)>1 then
     begin if par[2]='R' then switch:='#' end
    else switch:='?';
   case Par[1] of
    '?','-': Help(''); {- added to respect unix --help even without the help}
    'H': if length(par)>1 then
          if par[2]='U' then
           begin
            h_unrefracted:=ss2r(par,3);
            de_refract:=false;
           end
          else h:=par2r(par);
    '#','N': if length(par)>1 then
           begin
          if not BSC then
           if not DbInitSearch(Loc_bsc,'b5_int.dbf') then
            help(
'The Bright Star Catalogue not found in current directory or directories:'+cl+
Loc_bsc)
           else
            begin
             BSC:=true;
             na:= DbFieldNum('rh2');
             ndec:=DbFieldNum('dd2');
             nname:=DbFieldNum('name');
            end;
          hr:=1;
          if par[1]='H' then
           begin
            hr:=round(Par_num(copy(Par,2,length(par)-1)));
            if not DbRead(hr,rec) then
             help('There is no such number in the Bright Star Catalogue: '
                   +SI(4,HR));
           end
          else if length(par)>6 then
           begin
            if pos('_',ParVal)>0 then
             begin
              sname:=ItemStrD('_',1,ParVal);
              nflam:=I_S(sname);
              Greek_given:= nflam=0;
              cname:=ItemStrD('_',2,ParVal);
              pname:='_';
             end
            else
             begin
              sname:=copy(ParVal,1,3);
              cname:=copy(ParVal,5,3);
              pname:=par[4];
              part_given:=true;
             end;
            hr:=1;
            if not DbRead(hr,rec) then;
            repeat
             if copy (rec[nname],8,3)=cname then
              if Greek_given then
               begin
                found:=copy(rec[nname],4,3)=sname;
                if part_given then found:= found and (rec[7]=pname);
               end
              else
               begin
                found:=I_S(copy(rec[nname],1,3))=nflam;
                if part_given then found:= found and (rec[7]=pname);
               end;
             inc(hr);
            until found or not DbRead(hr,rec);
            if not found then help(+sname+pname+cname+'  not found');
           end;
          DbDisp(hr);
          writeln;
          val(__cvtstr(rec[na],_rem_white_str),hrhh,j);
          val(__cvtstr(rec[na+1],_rem_white_str),hrmm,j);
          val(__cvtstr(rec[na+2],_rem_white_str),hrss,j);
          tt:=hrss-(hrss div 10)*10;
          hrss:=hrss div 10;
          if Bab2Dec(1,hrhh,hrmm,hrss,tt,ra,'H') then;
          ra:=ra*15;
          val(__cvtstr(rec[ndec],_rem_white_str),hrdd,j);
          val(__cvtstr(rec[ndec+1],_rem_white_str),hrmm,j);
          val(__cvtstr(rec[ndec+2],_rem_white_str),hrss,j);
          tt:=hrss-(hrss div 10)*10;
          hrss:=hrss div 10;
          if pos('-',rec[ndec])>0 then
           begin
            hrdd:=abs(hrdd);
            if Bab2Dec(-1,hrdd,hrmm,hrss,tt,decl,'D') then
           end
          else
            if Bab2Dec(1,hrdd,hrmm,hrss,tt,decl,'D') then;
         end
         else help('');
    'R': if length(par)>2 then
          begin
           if par[2]='A' then
            begin
             par:=copy(par,3,length(par)-2);
             val(par,ra,code);
             if code > 0 then
              if BabS2Dec(Par,hrhh,hrmm,hrss,tt,ra,'H') then
               ra:=ra*15
              else
               if not BabS2Dec(Par,hrhh,hrmm,hrss,tt,ra,'D') then
                help(
   'RA gives no valid right ascension (either degrees or hh:mm:ss): '+par);
            end;
           if par[2]='H' then
            begin
             Relative_Humidity:=ss2r(par,3);
             if Relative_Humidity<0.01 then
              help('RH gives no valid relative humidity: '+par);
            end;
          end
         else
          help(par+' gives no value!');
    'D': if length(par)>1 then
          if par[2]='E' then
           begin
            if length(par)>2 then
             begin
              par:=copy(par,3,length(par)-2);
              val(par,decl,code);
              if code > 0 then
               if BabS2Dec(Par,hrhh,hrmm,hrss,tt,decl,'D') then
               else
                help(
   'De gives no valid declination (either degrees or dd:mm:ss): '+par);
             end
            else
                help(
   'De gives no declination (either degrees or dd:mm:ss): '+par);
           end
          else if not Par_Date then help('');
   'P': if length(par)>2 then
          case par[2] of
          'R':
            begin
             pressure_db_101325:=ss2r(par,3);
             if pressure_db_101325<0.02 then
              help('PR gives no valid pressure divided by 1013.25 Pa: '+par);
            end
          else
           help(par+' is no known parameter!')
          end
        else
         help(par+' gives no value!');
   'C': if length(par)>1 then
          if (par[2]='T') and (length(par)>2) then
           begin
            Celsius_temp:=ss2r(par,3);
            if Celsius_temp<-273 then
             help('CT gives no valid Celsius temperature / 1 K: '+par);
           end
          else
           Planet_No:=par2i(par)
        else
         help(par+' gives no value!');
   'E': PrecDate:=ss2r(par,3);
   'F': if not Par_Fi then help('');
   'L': if length(Par)>2 then
         case  Par[2] of
          'B': Loc_BSC:=copy(Par,3,length(par)-2);
          {'E': Loc_obj:=copy(Par,3,length(par)-2);}
         else
          if not Par_Lambda then help(' ')
         end
        else
         if not Par_Lambda then help(' ');
{   'T': if not Par_Time then help('');}
   'U': if not Par_UT then help('');
   else help('');
   end;
  end;

if t_UT<0 then t_ut_sig:='-';
hh:=12; mi:=0; ss:=0; hs:=0;
Get_JD;
if not tt_UT_set then tt_ut:=tt_UTC_in_days(JD);


if de_refract then
 if h>=-1 then  {two steps of iteration should perhaps do}
  begin
   h_unrefracted:=h-(refract_d(h,
    pressure_db_101325,Celsius_temp,Relative_humidity)-h);
   h_unrefracted:=h_unrefracted-(refract_d(h_unrefracted,
    pressure_db_101325,Celsius_temp,Relative_humidity)-h);
  end
 else h_unrefracted:=h
else
 if h_unrefracted>-2 then
   h:= refract_d(h_unrefracted,
    pressure_db_101325,Celsius_temp,Relative_humidity)
 else h:= h_unrefracted;

if Planet_No=-1 then
 begin
  h_geoc:=h_unrefracted;
  if PrecInit(PrecDate,YYYYreal(JD)) then;
  Precess(RA,Decl);
 end
else
 begin
  Planet_pos;
  if (Planet_No=10) and (h>=-0.3) then  {raising height by lunar parallax}
   begin
    h_geoc:=deg_to_rad(h_unrefracted);
    r:=rg-sin(h_geoc);
    h_geoc:=rad_to_deg(h_geoc + arctan(cos(h_geoc)/r));
   end
  else
   h_geoc:=h_unrefracted;
  writeln(cl,'The computed point is: ',Planet_Names[Planet_No]);
 end;

if not h2t(h_geoc,Decl,A,T,whystr) then
 begin
  writeln('The point is ',whystr,' than the given angular height');
  halt;
 end;

h2Air_Mass(h);

if h>-1 then
 writeln(cl,
'The point gets to the given (refracted) angular height of:',h:6:2,' degrees',cl,
'that is to the            (unrefracted) angular height of:',h_unrefracted:6:2,' degrees')
else
 writeln(cl,
'The point is (unvisibly) at the angular height of',h:6:2,' degrees');

RA:=RA/15;
T1:=12+(RA-T-SM)/Sidereal2SolarTime;
if T1>24 then T1:=T1-24/Sidereal2SolarTime;
if T1< 0 then T1:=T1+24/Sidereal2SolarTime;
T2:=12+(RA+T-SM)/Sidereal2SolarTime;
if T2>24 then T2:=T2-24/Sidereal2SolarTime;
if T2< 0 then T2:=T2+24/Sidereal2SolarTime;
if Planet_No=-1 then
 writeln(
'when its hour angle is minus or plus ',T:6:2,' hours',cl,
'that is at the local sidereal time of',Bas_Int(RA-T,24):6:2,' hours',cl,
'                                   or',Bas_Int(RA+T,24):6:2,' hours',cl,
'that is at the civic (UTC',t_ut_sig,abs(t_UT):4:1,
                              ' h) time of',T1:6:2,' hours ( = ',h_min_s(T1),' )',cl,
'                                       or',T2:6:2,' hours ( = ',h_min_s(T2),' )',cl2,

'Azimuth of the point is plus or minus:          ',
      Bas_Int(180-a,180):6:2,' degrees',cl,
'(from the south, i.e., "astronomical" azimuth:  ',a:6:2,' degrees)')
else
 begin
  refine_pos(T1,-1);
  refine_pos(T1,-1);
  if Planet_No=10 then refine_pos(T1,-1);
  writeln('when',cl,
 'its hour angle is           -',T:5:2,' hours',cl,
 'that is at the local sidereal time of',Bas_Int(RA-T,24):6:2,' hours',cl,
 'that is at the civic (UTC',t_ut_sig,abs(t_UT):4:1,
                               ' h) time of',T1:6:2,' hours ( = ',h_min_s(T1),' )',cl,
 'Azimuth of the point is :                       ',
      Bas_Int(180-a,180):6:2,' degrees',cl,
 '(from the south, i.e., "astronomical" azimuth: -',a:6:2,' degrees)');

  refine_pos(T2,1);
  refine_pos(T2,1);
  if Planet_No=10 then refine_pos(T2,1);
  writeln('or when',cl,
 'its hour angle is            ',T:5:2,' hours',cl,
 '           the local sidereal time is',Bas_Int(RA+T,24):6:2,' hours',cl,
 '           the civic (UTC',t_ut_sig,abs(t_UT):4:1,
                               ' h) time of',T2:6:2,' hours ( = ',h_min_s(T2),' )',cl,
 'Azimuth of the point is :                      -',
      Bas_Int(180-a,180):6:2,' degrees',cl,
 '(from the south, i.e., "astronomical" azimuth:  ',a:6:2,' degrees)');
 end;

 if h>=0 then
writeln(cl,
'Air mass (mass of a column of air in this direction',cl,
'relative to the mass of the vertical column of air) is: ',Air_mass:6:2);


if BSC then DbClose
end.
