 {~ Writes Julian dates (or MJDs) into a dBase file ~}

program Time2JD;
uses dos,graph,         {Borland Turbo Pascal units}
     DbBridge,un_str_m,params,solar_ut,angles_o,planets,str_num;
                        {author's units}

const
h10= cl+
'Time2JD <name of a dBase file> [j<output Julian date field>] [m]'+cl+
' [ { df<Date field> | { fdy<Year field> fdm<Month field.> fdd<Day field> } } ]'+cl+
' [n]'+cl;
h15=
' [ { t<Time field> | { fth<Hour field> ftm<Minute f.> [fts<Second field>]} } ]'+cl+
' [ { {[UT#] [UTF[9]]} | fu<T-UTC field> } | { l# [s] } ]  ra# de# '+cl2;

h20=
' writes the Julian date into a given dBase file.'+cl+
' Default values are /dDATE, /jJD, no Time-field (/t)'+cl+
' and UT0 (no difference of the used time and the Universal Time Coordinated;'+cl+
' the difference is assumed to be given in hours or as hh:mm:ss.ss)'+cl2;

h30=
' m asks for a Modified Julian Date (MJD=JD-2 400 000.5)'+cl+
' n says that the Date format is YYMMDD instead of YYYYMMDD.'+cl+
' l gives longitude (local time is then assumed),'+cl+
' s says that it is a true Solar time, not a mean one.'+cl;
h31=
' RA and De give equatoreal coordinates of a fixed star, for which'+cl+
'  the heliocentric correction will be applied'+cl+
'  (plus or minus up to five hundred seconds)'+cl2;

h35=
'The Date field can contain fractions of a day, with or without a period'+cl;
h40=
' after DD - in this case the /t switch should miss.'+cl+
' Instead of Date field, Year, Month and Day fields can be given,'+cl+
' instead of Time field, Hour, Minute (and perhaps Second) fields can be used.'+cl;
h50=
' In these cases, repeated values may be left blank (e.g., month can appear'+cl+
' just when it is different from the preceding record).'+cl;
h60=
'The Time field followed by :d means time in fractions of a day,'+cl+
'UTF means: summertime (+1 h) from the last March Sunday to the last October'+cl+
' Sunday, UTF9 to the last September Sunday'+cl2;

h70=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;

 TimeToo  : boolean = false;
 TimeInDays: boolean = false;
 DateFrac : boolean = true;
 NoJuldate: boolean = false;
 DateField: string ='DATE';
 JDField  : string = 'JD';
 TimeField: string = '';
 Find_T_UTC: boolean = false;
 In_October: boolean = true;
 m9 : byte =9;  {Last month with the full summertime}
 m10: byte=10;  {Autumn month of the summertime end}
 Century  : integer = 0;
 LYear    : byte = 4;
 ModiJD   : real =0;
 hour     : word=0;
 minutes  : real=0;
 seconds  : real=0;
 Nhour    : byte=0;
 Nmin     : byte=0;
 Nsec     : byte=0;
 Nyear    : byte=0;
 Nmonth   : byte=0;
 Nday     : byte=0;
 NT_UTc   : byte=0;
 day_r    : real=0;
 true_local:boolean=false;
 RA_fix   : real=1e10;
 Decl_fix : real=1e10;
 HelCor   : boolean=false;

var
 jfor,Start_frac : byte;
 j,day        : integer;
 juldate,Time : real;
 parSW1,parSW2  : char;
 NDatum,LDatum,NJD,LJD,DJD,NTime,ValidPlaces : byte;
 NoR,Year,Month,hours,mins,secs,tics     : word;
 aux_str : string[2];


 procedure help;
  begin
   write(h10,h15,h20,h30,h31);
   c_o_halt;
   write(h35,h40,h50,h60,h70);
   halt;
  end;

 function NumDbField(NameField:string):byte;
  var aux_num:byte;
  begin
   aux_num:=DbFieldNum(NameField);
   if aux_Num=0 then begin writeln(
'The file ',paramstr(1),' contains no field ', NameField,'!'); halt end
   else NumDbField:=aux_num;
  end;

 procedure GetFullJulDate;
  var add_t_UT: byte; DoW_num: byte;
  begin
   add_t_UT:=0;
   if Find_T_UTC then
    begin
     DoW_num:=DoWN(JulDate+ModiJD+(Time-t_UT)/24,t_UT);
     add_t_UT:=0;
     if In_October then
      case month of
      4..9: inc(add_t_UT);
      10  : if (DoW_Num<7) and (day-DoW_Num<25) then inc(add_t_UT)
            else if DoW_Num=7 then
                  if day<25 then inc(add_t_UT)
                  else if Time<3 then inc(add_t_UT);
        3 : begin
             add_t_UT:=1;
             if (DoW_Num<7) and (day-DoW_Num<25) then dec(add_t_UT)
             else if DoW_Num=7 then
                   if day<25 then dec(add_t_UT)
                   else if Time<3 then dec(add_t_UT);
            end;
      end
     else
      case month of
      4..8: inc(add_t_UT);
       9  : if (DoW_Num<7) and (day-DoW_Num<24) then inc(add_t_UT)
            else if DoW_Num=7 then
                  if day<24 then inc(add_t_UT)
                  else if Time<3 then inc(add_t_UT);
        3 : begin
             add_t_UT:=1;
             if (DoW_Num<7) and (day-DoW_Num<25) then dec(add_t_UT)
             else if DoW_Num=7 then
                   if day<25 then dec(add_t_UT)
                   else if Time<3 then dec(add_t_UT);
           end;
      end
    end;
   JulDate:=JulDate+(Time-(t_UT+add_t_UT))/24
  end;

 function NewValW(fromstr:string; var r:word):boolean;
  begin
   NewValW:=false;
   if copy(fromstr,1,2)<>'  ' then
    begin
     val(__cvtstr(fromstr,_rem_white_str),r,j);
     if j>0 then NoJulDate:=true
     else NewValW:=true;
    end
  end;

 procedure NewValR(fromstr:string; var r:real);
  begin
   if fromstr<>'  ' then
    begin
     val(__cvtstr(fromstr,_rem_white_str),r,j);
     if j>0 then NoJulDate:=true;
    end;
  end;

begin
 t_UT:=0;
 if paramcount<1 then help;
 if not DbInit(paramstr(1)) then begin writeln(
'No dBase file ',paramstr(1),' found!'); help end;
 if paramcount > 1 then  for jfor:=2 to paramcount do
   begin
    Par:=paramstr(jfor);
    if (Par[1]='/') or (Par[1]='-') then Par:=copy(Par,2,length(par)-1);
    Par:=__cvtstr(Par,_To_UpCase_Str);
    case Par[1] of
     '?','-': Help; {- added to respect unix --help even without the help}
     'D': case Par[2] of
           'F': if length(par)>2 then DateField:=copy (Par,3,length(Par)-2);
           'E': if not Par_Decl then halt;
          end;
     'F': if length(par)>2 then
           begin
            parSW1:=par[2];
            parSW2:=par[3];
            par:=copy(par,4,length(Par)-3);
            case parSW1 of
             'T': begin
                   TimeToo:=true;
                   case parSW2 of
                    'H': Nhour:=NumDbField(par);
                    'M': Nmin:=NumDbField(par);
                    'S': Nsec:=NumDbField(par);
                   end;
                  end;
             'D': begin
                   DateField:='';
                   case parSW2 of
                    'Y': NYear:=NumDbField(par);
                    'M': NMonth:=NumDbField(par);
                    'D': NDay:=NumDbField(par);
                   end;
                  end;
             'U': if parSW2='T' then NT_UTc:=NumDbField(par);
            end;
           end;
     'J': JDField:=copy (Par,2,length(Par)-1);
     'L': begin
           if not Par_Lambda then halt;
           T_UT:=12*Lambda/pi;
          end;
     'M': ModiJD:=2400000.5;
     'N': begin
           LYear:=2;
           Century:=1900;
          end;
     'R': if Par[2]='A' then if not Par_RA then halt;
     'S': true_local:=true;
     'T': begin
            TimeField:=copy (Par,2,length(Par)-1);
            If ItemCountD(':',TimeField)>1 then
             begin
              TimeInDays:=ItemStrD(':',2,TimeField)='D';
              TimeField:=ItemStrD(':',1,TimeField);
             end;
            TimeToo:= true
          end;
     'U': begin
           {the two-character switch is skipped}
           if length(Par)>2 then
           if Par[3]='F' then
            begin
             Par:=copy(Par,3,length(Par)-2);
             Find_T_UTC:=true;
             if length(Par)>1 then
              begin if Par[2]='9' then
               In_October:=false
              end
             else
              In_October:=true;
            end
           else
            if not Par_UT then halt;
          end;
    end; {of case}
   end; {of for}

 if DateField<>'' then
  begin
   NDatum:=NumDbField(DateField);
   LDatum:=DbInfo[NDATUM].len;
   if LDatum>LYear+4 then DateFrac:=true
   else DateFrac:=false;
  end
 else
  if (NYear=0) or (NMonth=0) or  (NDay=0) then
   begin
    writeln('Not all of Year Month and Day fields have been given.');
    halt
   end;

 NJD:=NumDbField(JDField);
 LJD:=DbInfo[NJD].len;
 DJD:=DbInfo[NJD].dec;

 if TimeToo then
  if TimeField<>'' then
   NTime:=NumDbField(TimeField)
  else
   if (NHour=0) or (NMin=0) then
    begin
     writeln('Not both Hour and Minute fields have been given.');
     halt
    end;

 if (RA_fix<1e9) and  (Decl_fix<1e9) then HelCor:=true;

 NoR:=1;
 while DbRead(NoR,rec) do
  begin
   NoJuldate:=false;
   if DateField<>'' then
    begin
     val(copy(rec[NDatum],1,LYear),year,j);
     if j>0 then writeln(
 'record ',NoR,' has a non-numeric character within Year at position ',j);
     inc(year,Century);

     val(copy(rec[NDatum],LYear+1,2),month,j);
     if j>0 then writeln(
 'record ',NoR,' has a non-numeric character within Month at position ',j);

     val(copy(rec[NDatum],LYear+3,2),day,j);
     if j>0 then writeln(
 'record ',NoR,' has a non-numeric character within Day at position ',j);
    end
   else      begin
     if NewValW(rec[NYear],year) then
      inc(year,century);
     aux_str:=rec[NMonth];
     if DbInfo[NMonth].len=1 then
      aux_str:=' '+aux_str;
     if NewValW(aux_str,month) then;
     NewValR(rec[NDay],day_r);
     day:=trunc(day_r);
    end;

   juldate:=jd0(year,month,day)-ModiJD;

   if NT_UTc>0 then
    begin
     if copy(rec[NT_UTc],1,2)<>'  ' then
      if __2TimeStr(rec[NT_UTc],Hours,Mins,Secs,Tics) then
        T_UT:=hours+mins/60+secs/3600+tics/360000;
    end
   else
    if true_local then
     begin
      Ecl_to_Equ(Sun_Long(juldate+modiJD),ra,decl);
      T_UT:=12*lambda/pi + Get_SM(juldate+modiJD+0.5,0) - RA/15;
      if T_UT>12 then T_UT:=T_UT-24;
     end;

   if TimeToo then
    if TimeField<>'' then
     begin
      val(__cvtstr(rec[NTime],_rem_white_str),Time,j);
      if j>0 then
       if __2TimeStr(rec[NTime],Hours,Mins,Secs,Tics) then
        Time:=hours+mins/60+secs/3600+tics/360000
       else
        begin
         NoJuldate:=true;
         writeln(
'record ',NoR,' has a queer character within Time at position ',j)
        end;
      if TimeInDays then Time:=Time*24;
      GetFullJulDate;
     end
    else
     begin
      if NewValW(rec[NHour],hour) then;
      NewValR(rec[NMin],minutes);
      if Nsec>0 then
       NewValR(rec[Nsec],seconds);
      Time:=hour+minutes/60+seconds/3600;
      GetFullJulDate;
     end
   else
     if DateFrac then
      begin
       ValidPlaces:=LDatum-LYear-4;
       if rec[NDatum][LYear+5]='.' then
        begin
         dec(ValidPlaces);
         Start_frac:=LYear+6;
        end
       else
        Start_frac:=LYear+5;
       val(copy(rec[NDatum],Start_frac,ValidPlaces),Time,j);
       if j>0 then
        begin
          ValidPlaces:=j-1;
          val(copy(rec[NDatum],Start_frac,ValidPlaces),Time,j)
        end;
       if Time>=1 then for jfor:=1 to ValidPlaces do Time:=Time/10;
       GetFullJulDate;
      end
     else
      if frac(day_r)>0 then
       begin
        Time:=frac(day_r)*24;
        GetFullJulDate;
       end;

   if HelCor then
     JulDate:=JulDate +
      Heliocentric_correction_s(JulDate+ModiJD,RA_fix,Decl_fix)/86400;
   str(JulDate:LJD:DJD,rec[NJD]);
   if NoJuldate then rec[NJD]:='';
   if DbWrite(NoR,rec) then;
   inc(NoR);
  end;
 DbClose;
end.

