program wedge256; {Jan Hollan}

uses Graph,DOS,str_num;
const
cl=#13#10; SVGA:boolean=TRUE; width:byte=15;
h1=cl+
'Wedge256 { V | S | #} tries to show a wedge of available steps of gray'+cl+cl+
' -- with parameter V it starts in highest available VESA mode (in case'+cl+
' that the Free Pascal Compiler has been used), otherwise in VGA mode,'+cl;
h1a=
' with S in 256 color mode (but just 64 shades of gray) in 320x200 screen,'+cl+
' with # in some SVGA mode (both these parameters hold for the Borland'+cl+
'        Turbo Pascal compiler):'+cl;
h2=
'0 means 320x200-dot screen (like S) -- just this should always work,'+cl+
'1 is 640x400, 2 is 640x480,'+cl+
'3 is 800x600 and'+cl+
'4 is 1024x768 (needs 1 MB of video memory!).'+cl+cl;
h3=
'No parameter, ?, /? or -? will show just this help.'+cl+
'Quite a few video cards lock up the computer with # over 0,'+cl+
'so that it has to be restarted by a Reset button!!!'+cl+cl;
h4=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;


const InGraphMode:boolean=true;
var  i,j:integer; wrong_cod:integer; TH:byte;
  GraphDriver,GraphMode{,LowGDMode,HighGDMode}:integer;
  InGraphicsMode : boolean; { Flags initialization of graphics mode }
  PathToDriver   : string;  { Stores the DOS path to *.BGI & *.CHR }
  AutoDetectPointer : pointer;
  w,MaxColor,MaxX,MaxY,ix,iy:word;
  aux_s:string;

procedure fail(message:string);
 var errorcode:integer;
 begin
  ErrorCode := GraphResult;
  if (ErrorCode <> grOK) or (message='') then
   begin
    if InGraphMode then closegraph;
    if (message<>'') then
     writeln(message,':  ',ErrorCode,GraphErrorMsg(ErrorCode),cl);
    write(h1,h1a,h2,h3,h4);
    halt
   end;
 end;

{$F+}
function DetectSVGA256 : integer;

 { Detects ?VGA or MCGA video cards }
 (*var
   DetectedDriver : integer;
   SuggestedMode  : integer;
 begin
   DetectGraph(DetectedDriver, SuggestedMode);
   writeln(DetectedDriver:3,SuggestedMode:3);
   if (DetectedDriver = VGA) or (DetectedDriver = MCGA) then
     DetectSVGA256 := SuggestedMode    { Default video mode = 0 }
   else
    begin
     DetectSVGA256 := grError; { Couldn't detect hardware }
     fail('DetectSVGA256 fails');
    end;
 *)

 begin
 DetectSVGA256 :=0; {:this sets to 320x200, 2 to 640x480, 3 to 800x600}
 end; { DetectVGA256 }
{$F-}

procedure Set256Palette;
 type
   RGBColor   = record
                  R, G, B : byte;
                end;
   VGAPalette = array[0..255] of RGBColor;
 var
   VGAPal   : VGAPalette;

 procedure SetDACBlock(Start, Count : integer; var Pal : VGAPalette);
  var
    Regs : Registers;
  begin
    with Regs do
    begin
      AH := $10;
      AL := $12;
      BX := Start;
      CX := Count;
      ES := Seg(Pal);
      DX := Ofs(Pal);
    end;
    Intr($10, Regs);
  end;

 begin
     for w:=0 to MaxColor do
       with VGAPal[w] do begin R:=w shr 2; G:=w shr 2; B:=w shr 2 end;
     SetDACBlock(0, 256, VGAPal);
 end; { Set256Palette }

begin
 PathToDriver := getenv('BGI');
 if ParamCount=0 then fail('');
 Aux_s:=ParamStr(1);
 if UpCase(aux_s[1])='V' then SVGA:=false;
 if pos('?',Aux_s) > 0 then fail('');
 if SVGA then
  begin
    AutoDetectPointer := @DetectSVGA256;   { Point to detection routine }
    GraphDriver := InstallUserDriver('SVGA256', AutoDetectPointer);
    fail('SVGA256 not installed');
  end;
 GraphDriver := 0;  GraphMode:=0;
 InitGraph(GraphDriver, GraphMode, PathToDriver);
 fail('InitGraph protest');
 InGraphMode:=true;
 if SVGA then
  begin
   (*  GetModeRange(GraphDriver,LowGDMode,HighGDMode); *)
   {HighGDMode is always 1??? in spite of GetMaxMode=4!!!}
   val(Aux_s,j,wrong_cod);
   if wrong_cod=0 then
    if j<=GetMaxMode then GraphMode:=j
    else fail('Maximum number of mode is '+SI(3,GetMaxMode));
   SetGraphMode(GraphMode);
   fail('SetGraphMode protest on'+SI(3,GraphMode));
  end;
 MaxColor := GetMaxColor;  { Get the maximum allowable drawing color }
 MaxX := GetMaxX;          { Get screen resolution values }
 MaxY := GetMaxY;
 if (GraphMode>10) then SVGA:=false;
 if SVGA then
  Set256Palette
 else
  begin
   for i := 0 to MaxColor do
    begin
     if MaxColor<16 then j:=i*4 else j:=i;
     SetRGBPalette(i,j,j,j);
    end;
   if MaxColor<16 then
    for i:=0 to MaxColor do SetPalette(i,i);
    {for VESA mode, the above line would spoil the gray palette again!?}
  end;
 setcolor(MaxColor);
 TH:=round(1.2*TextHeight('1'));
 outtextxy(20,20,
           'driver:'+SI(2,GraphDriver)+'='+GetDriverName+
           ', mode:'+SI(2,GraphMode)+'='+GetModeName(GraphMode)+
           ', out of:'+SI(2,getmaxmode));
 outtextxy(20,20+TH,
           ' Low:'+SI(2,0)+'='+GetModeName(0)+
           ', Hi:'+SI(2,GetMaxMode)+'='+
                GetModeName(GetMaxMode));
 i:=trunc(ln(MaxColor)/ln(10));
 outtextxy(20,20+2*TH,
           'MaxColor:'+SI(i+2,GetMaxColor)+
           '  MaxX:'+si(4,MaxX)+
           '  MaxY:'+si(4,MaxY));
 j:=TH*3 + 25;
 if GraphMode=0 then width:=7;
 for i:=0 to MaxColor do
  begin
   setcolor(i);
   setfillstyle(solidfill,i);
   ix:=16*(i mod 16);
   iy:=(width+1)*(i div 16)+j;

   rectangle(ix,iy,ix+15,iy+width);
   floodfill(ix+1,iy+1,i)
  end;
 iy:=iy + width*3;
 outtextxy(20,iy,'Pres <ENTER> to quit.');
 readln(Aux_s);
 closegraph;
end.
