 {~ Reading (in future perhaps writing into and creating) FITS tables/images ~}

Unit FtBridge;
     {written according to Milan Dr\'a\v sil, by Jan Hollan}
(*  Copyright (C) 1999 Jan Hollan;
  by "program" this unit is further meant.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)


Interface
uses Dos,
     Dbbridge,un_str_m;

const
 Ft_max_fields=max_fields{60};     {Maximum allowed number of fields}
 Ft_max_length=1024;   {Maximum allowed length of record}
 FtNoWriteHeading:boolean=true;
 FtImage:boolean=false;

type Ft_inforec = record
                field : string[10];
                typ   : char;
                len   : byte;
                dec   : byte;
                start : word;
               end;
 FtImLine1Type = array[1..2048] of byte;
 FtImLine2Type = array[1..1024] of word;

{     FtStr   = array[1..Ft_max_fields] of string; -> DbStr instead}


var FtInfo    : array[1..Ft_max_fields] of Ft_inforec; {field description}
    FtRecl    : word;                    {record length}
    FtOff     : longint;                 {Start of records}
    FtNuRec   : longint;                 {Number of Records}
    FtImBit   : byte;                    {bits per pixel, for images}
    FtFileName: PathStr;                 {Name of File, incl. path}
    FtYY,FtMM,FtDD: byte;
    FtImLine1:  FtImLine1type;
    FtImLine2:  FtImLine2type absolute FtImLine1;

    Ft_rec    : DbStr;                         {one record - loose}
               {this is the only coupling to DbBridge}

    Ft_OneRec : array [1..Ft_max_length] of char;    {one record - dense}
    FtDocFile : text;
const
    FtNuField : word=0;                    {Number of Fields}

Function FtInit(f : string;TableNoWanted:byte) : Boolean;
           {opening an existing file - False if not found}
                          {TableNoWanted=0 means an image}
function  FtInitSearch
           (StandLoc:string;SomeFileName:PathStr;TableNoWanted:byte):boolean;
           {adding suffix .fit if needed and then, if no directory is given
            in FtFileName and file is not found in the current one, searching
            through the StandLoc (DirString: series of directories
            delimited by ;).
            If still not found (or FtInit fails),
            writing a message on the screen.}
Function  FtRead(RecNo : longint; var Ft_rec : DbStr) : boolean;
           {reading a record number RecNo}
Function FtReadIm(recno : longint; var FtImLine : FtImLine1Type) : boolean;
           {the same for a line of image: the line is of byte type,
            but a word type FtImLine2 is same-placed if the actual
            parameter names FtImLine1}
Function  FtFieldNum(F_Nam:string):byte;
           {gives the number of the field F_Nam; 0 when fails}

Procedure FtDisp(recno:longint);
           {displays a record number RecNo on the screen}


(*  {The following group of procedures are analogues of those in DbBridge,
     but they wait yet to be adatped to work for FITS:}
Procedure FtNewField(F_Nam:string;F_Typ:char;F_len,F_dec:byte);
           {adding a field to a file to be created}
Function  FtCreate(f: string): Boolean;
           {creating a file with fields defined by the above procedure}
Procedure FtWrNum(FieldNum: byte; Value: real);
           {filling the corresponding field with a number}
Function  FtWrite(recno : longint; var Ft_rec : DbStr) : boolean;
           {writing a record number RecNo;
                                    RecNo=0 asks for appending a new record}
Procedure FtStruDescr(f:string);
           {creates a file describing structure of another FITS file}
Procedure FtDescField(fn:string;ft:char;flen,fdec:byte;fdesc:string);
           {adds a record to this file}
*)


Procedure FtClose;
           {closing the file and setting the current date, if needed}


Implementation

type hearec = record
               FtSign,YY,MM,DD   :byte;
               reccount : longint;
               offset   : word;
               reclen   : word;
               n3       : array [1..5] of longint;
              end;
type idrec  = record
               field: array[1..10] of char;
               n1   : byte;
               typ  : char;
               n2   : longint;
               len  : byte;
               dec  : byte;
               n3   : array [1..14] of byte
              end;


var i,j : byte;
    v1      : hearec;
    f1      : file of hearec;
    v2      : idrec;
    f2      : file of idrec;
    fd   : file ;                {to have a possibility to access the file  }

const
 head:string[8]='        '; head5:string[5]='     ';
 FtBlockLength=2880;
 FtAppend:boolean=false;
 FtOpen:boolean=false;
 FtOK : boolean=true;

Function FtInit(f : string;TableNoWanted:byte) : Boolean;
var
 blocks_read:word;
 line_arr:array[1..36,1..80] of char;
 aux_st:string;
 known,i_f:byte;
 OK:integer;
 NumRead:integer;
 TableNo,lineNo:byte;
 NoWriteHeading,FtNoWriteHeading_orig:boolean;
const LI1:longint=1;

function la2i(which:byte):longint;
 var i1:longint; i2:integer;
 begin
  aux_st:='';
  for i:= 10 to 31 do
   if line_arr[lineNo,i]<>' ' then aux_st:=aux_st+line_arr[lineNo,i];
  val(aux_st,i1,i2);
  la2i:=i1;
  if i2=0 then
   known:=known+which;
 end;

procedure Next_Block;
 begin
  BlockRead(fd,line_arr,1,NumRead);
  If NumRead=1 then
   begin
    inc(blocks_read);
    lineNo:=1;
   end;
 end;

procedure WriteLine;
var
 line:string;
begin
 if FTNoWriteHeading then exit;
 line:='';
    for i:=1 to 80 do
     line:=line+line_arr[lineNo,i];
 writeln(FtDocFile,__cvtstr(line,_rem_trail_white_str));
end;


Function FindEnd:boolean;
begin
 FindEnd:=false;
 repeat
  inc(lineNo);
  if lineNo=37 then
   begin
    Next_Block;
    if NumRead=0 then exit;
   end;
  for i:=1 to 8 do
   head[i]:=line_arr[lineNo,i];
  WriteLine;
 until (head='END     ');
 FindEnd:=true;
 if not FTNoWriteHeading then Writeln(FtDocFile,#13#10);
end;

procedure FillHead;
begin
 inc(lineNo);
 for i:=1 to 8 do
  head[i]:=line_arr[lineNo,i];
 WriteLine;
end;

begin
 FtInit:=False;
 if FtOpen then exit;
 blocks_read:=0;
 TableNo:=0;
 FtNoWriteHeading_orig:=FtNoWriteHeading;
 NoWriteHeading:=FTNoWriteHeading_orig;
 if pos('.',f)=0 then f:=f+'.fit';
 FtFileName:=f;
 If TableNoWanted=0 then FtImage:=true;

{$I-}
 assign(fd,f);
 FileMode:=0; {will open the files by reset as Read Only}
 reset(fd,FtBlockLength);
 if ioresult<>0 then
  begin
   {$I+}
   Exit;
  end;
{$I+}

 if not FTNoWriteHeading then
  Writeln(FtDocFile,'Heading of the whole FITS file ',f,' reads:',#13#10);
 Next_Block;      {Description of the whole file:}
 lineNo:=0;
 if NumRead=0 then exit;
 FillHead;
 if head<>'SIMPLE  ' then Exit;
 if line_arr[lineNo,30]<>'T' then exit;
 FillHead;
 if head<>'BITPIX  ' then Exit;
 FtImBit:=la2i(1);
 if (FtImBit<>8) and (not FtImage) then exit;
 FillHead;
 if head<>'NAXIS   ' then Exit;
 if FtImage then
  begin
   if not la2i(1)=2 then Exit;
   FillHead;
   if head<>'NAXIS1  ' then Exit;
   FtRecl:=la2i(1);
   if FtImBit=16 then inc(FtRecl,FtRecl);
   FillHead;
   if head<>'NAXIS2  ' then Exit;
   FtNuRec:=la2i(2);
   if not FindEnd then Exit;
   FtOff:=LI1*FtBlockLength*blocks_read;
  end
 else
  begin
   if not la2i(1)=0 then exit;
   FillHead;
   if head<>'EXTEND  ' then Exit;
   if line_arr[lineNo,30]<>'T' then exit;
   if not FindEnd then exit;

   repeat
    inc(TableNo);
    if TableNoWanted>TableNo then
     begin
      NoWriteHeading:=FtNoWriteHeading_orig;
      FtNoWriteHeading:=true;
     end
    else
     begin
      FtNoWriteHeading:=NoWriteHeading;
      if not FTNoWriteHeading then
       Writeln(FtDocFile,'Heading of its table No',TableNo:3,' reads:',#13#10);
     end;
    seek(fd,blocks_read);
    Next_Block;              {Description of the single table:}
    if NumRead=0 then exit;
    known:=0;
    repeat
     for i:=1 to 8 do
      head[i]:=line_arr[lineNo,i];
     WriteLine;
     if head='NAXIS1  ' then
      v1.reclen:=la2i(1)
     else
      if head='NAXIS2  ' then
       v1.reccount:=la2i(2)
      else
       if head='TFIELDS ' then
        FtNuField:=la2i(4);
       {
       else
        if head='DATE    ' then
         begin
          aux_st:=line_arr[lineNo,12]+line_arr[lineNo,13];
          val(aux_st,FtDD,OK);
          if OK>0 then exit;
          aux_st:=line_arr[lineNo,15]+line_arr[lineNo,16];
          val(aux_st,FtMM,OK);
          if OK>0 then exit;
          aux_st:=line_arr[lineNo,18]+line_arr[lineNo,19];
          val(aux_st,FtYY,OK);
          if OK>0 then exit;
          known:=known+8;
         end;
       }
     inc(lineNo);
     if lineNo=37 then
      begin
       Next_Block;
       if NumRead=0 then exit;
      end;
    until known=7;

    FtRecl:=v1.reclen;
    FtNuRec:=v1.reccount;
    if FtNuField>Ft_max_fields then
     begin
      writeln('Too many fields (',FtNuField:3,')  maximum is',Ft_max_fields:3);
      Exit;
     end;
    known:=0;

    if TableNoWanted=TableNo then
     for i_f:= 1 to FtNuField do
      begin
       known:=0;
       repeat
        WriteLine;
        aux_st:=line_arr[lineNo,6];
        if line_arr[lineNo,7] in ['0'..'9'] then
         begin
          aux_st:=aux_st+line_arr[lineNo,7];
          if line_arr[lineNo,8] in ['0'..'9'] then
           aux_st:=aux_st+line_arr[lineNo,8]
         end;
        val(aux_st,i,OK);
        if i_f=i then
         begin
          for i:=1 to 5 do
           head5[i]:=line_arr[lineNo,i];
          if head5='TTYPE' then
           begin
            FtInfo[i_f].field:='';
            for i:=12 to 21 do
             FtInfo[i_f].field:=FtInfo[i_f].field+line_arr[lineNo,i];
            if pos(chr(39),FtInfo[i_f].field)>0 then
             FtInfo[i_f].field:=copy(FtInfo[i_f].field,1,
                                     pos(chr(39),FtInfo[i_f].field)-1);
            FtInfo[i_f].field:=__cvtstr(FtInfo[i_f].field,_rem_white_str);
            inc(known,1);
           end
          else
           if head5='TFORM' then
            begin
             i:=13;
             aux_st:=line_arr[lineNo,i];
             inc(i);
             while line_arr[lineNo,i] in ['0'..'9'] do
              begin
               aux_st:=aux_st+line_arr[lineNo,i];
               inc(i)
              end;
             val(aux_st,FtInfo[i_f].len,OK);
             case line_arr[lineNo,12] of
              'I': begin
                 FtInfo[i_f].typ  :='N';
                 FtInfo[i_f].dec  :=0;
                   end;
              'A': begin
                 FtInfo[i_f].typ  :='C';
                 FtInfo[i_f].dec  :=0;
                   end;
              'F': begin
                 FtInfo[i_f].typ  :='N';
                 inc(i);
                 aux_st:=line_arr[lineNo,i];
                 val(aux_st,FtInfo[i_f].dec,OK);
                   end;
             end;
             inc(known,2);
            end
           else
            if head5='TBCOL' then FtInfo[i_f].start:=la2i(4);
         end;
        inc(lineNo);
        if lineNo=37 then
         begin
          Next_Block;
          if NumRead=0 then exit;
         end;
       until known=7;
      end;

    dec(lineNo);
    if not FindEnd then exit;

    FtOff:=LI1*FtBlockLength*blocks_read;

    if TableNoWanted>TableNo then
     begin
      inc(blocks_read,(LI1*FtRecl*FtNuRec) div FtBlockLength);
      if (LI1*FtRecl*FtNuRec) mod FtBlockLength > 0 then
       inc(blocks_read,1);
      if FileSize(fd)<=blocks_read then exit;
     end;
   until TableNo=TableNoWanted;
  end;
 assign(fd,f);
 reset(fd,1);
 FtAppend:=false;
 FtOpen:=true;
 FtInit:=true;
end;

function FtInitSearch(StandLoc:string;SomeFileName:PathStr;TableNoWanted:byte):boolean;
 {adding suffix .fit if needed and then, if no directory is given
  in FtFileName and file is not found in the current one, searching
  through the StandLoc (DirString: series of directories delimited by ;).
  If still not found (or FtInit fails), writing a message on the screen.}
var FoundName: PathStr; FDir: DirStr; FName: NameStr; FExt: ExtStr;
begin
 FtInitSearch:=false;
 FSplit(SomeFileName,FDir,FName,FExt);
 if FExt='' then SomeFileName:=FDir+FName+'.fit';

 FoundName:=FSearch(SomeFileName,'');
 if FoundName<>'' then
  begin
   if not FtInit(FoundName,TableNoWanted) then
    begin
     writeln(
'the file >',FoundName,'< contains no ASCII table No.',TableNoWanted:3);
     exit
    end
  end
 else
  begin
   if FDir='' then
    FoundName:=FSearch(SomeFileName,StandLoc);
   if FoundName='' then
    begin
     writeln('FITS file ',SomeFileName,' not found');
     exit
    end
   else
    if not FtInit(FoundName,TableNoWanted) then
     begin
      writeln('the file >',FoundName,'< contains no ASCII table No.',TableNoWanted:3);
      exit
     end
  end;
 FtInitSearch:=true;
end;

Function FtCreate(f : string) : Boolean;
var Year,Month,Day,DoW:word;
const CR:byte=13;
begin
if (not FtOpen)
and FtOK {this is false if FtInfo field was not filled OK by FtNewFileld}
then
 begin
  if pos('.',f)=0 then f:=f+'.fit';
  FtFileName:=f;
  v1.FtSign:=3;
  GetDate(Year,Month,Day,DoW);
  v1.YY:=Year-1900;
  v1.MM:=Month;
  v1.DD:=Day;
  FillChar(v1.n3,20,0);
  FtRecl:=1;
  for i:=1 to FtNuField do inc(FtRecl,FtInfo[i].len);
  v1.reclen:=FtRecl;
  FtOff:=(FtNuField+1)*32+1;
  v1.offset:=FtOff;
  v1.reccount:=0;
  FtNuRec:=0;
  assign(f1,f);
  rewrite(f1);
  write(f1,v1);
  close(f1);

  v2.n1:=0;
  v2.n2:=0;
  FillChar(v2.n3,14,0);
  assign(f2,f);
  FileMode:=2; {will open the files by reset as writable}
  reset(f2);
  seek(f2,1);
  for i:= 1 to FtNuField do
   begin
    fillchar(v2.field,10,0);
    for j:=1 to length(FtInfo[i].field) do v2.field[j]:=FtInfo[i].field[j];
    v2.typ:=FtInfo[i].typ;
    v2.len:=FtInfo[i].len;
    v2.dec:=FtInfo[i].dec;
    write(f2,v2);
   end;
  close(f2);

  assign(fd,f);
  reset(fd,1);
  seek(fd,FtOff-1);
  BlockWrite (fd,CR,1);
  FtAppend:=false;
  FtOpen:=true;
  FtCreate:=true
 end
else
 FtCreate:=false
end;

Procedure FtNewField(F_Nam:string;F_Typ:char;F_len,F_dec:byte);
begin
If not (F_len>0) then FtOK:= False;
F_Typ:=UpCase(F_Typ);
If F_Typ='C' then F_dec:=0
else if F_Typ='L' then begin F_Len:=1; F_Dec:=0 end
     else if F_Typ='N' then
            begin if (F_Len>19) or (F_dec>17) then FtOK:=False end
          else if F_Typ='D' then begin F_len:=8; F_Dec:=0 end
               else FtOK:=False;
if FtOK then
 begin
  inc(FtNuField);
  with FtInfo[FtNuField] do
     begin
      if length(F_Nam)<10 then i:= length(F_Nam) else i:=10;
      field:='';
      for j:=1 to i do field:=field+UpCase(F_Nam[j]);
      typ:=F_Typ; len:=F_len; dec:=F_dec
     end;
 end
end;

Function FtRead(recno : longint; var Ft_rec : DbStr) : boolean;

var k: word;

begin
 if (recno>FtNurec) or (recno<1)
  then
   FtRead:=FALSE
  else
   begin
    FtRead:=TRUE;
    seek(fd,FtOff+(recno-1)*FtRecl);
    BlockRead (fd,Ft_OneRec,FtRecl);
    for i:=1 to FtNuField do
     begin
      k:=FtInfo[i].start;
      for j:=1 to FtInfo[i].len do
       begin
        Ft_rec[i][j]:=Ft_OneRec[k];
        inc(k);
       end;
      Ft_rec[i][0]:=chr(FtInfo[i].len);
     end;
   end;
end;

Function FtReadIm(recno : longint; var FtImLine : FtImLine1Type) : boolean;
begin
 if (recno>FtNurec) or (recno<1)
  then
   FtReadIm:=FALSE
  else
   begin
    FtReadIm:=TRUE;
    seek(fd,FtOff+(recno-1)*FtRecl);
    BlockRead (fd,FtImLine,FtRecl);
   end;
end;

Procedure FtWrNum(FieldNum: byte;Value:real);
begin
 str(Value:FtInfo[FieldNum].len:FtInfo[FieldNum].dec,Ft_rec[FieldNum]);
end;

Function FtWrite(recno : longint; var Ft_rec : DbStr) : boolean;

var k : word;

begin
 if recno>FtNurec then
  FtWrite:=FALSE
 else
  begin
    FtWrite:=TRUE;
    if recno=0 then
     begin
      if FileMode=0 then
       begin
        close(fd);
        FileMode:=2; {will open the files by reset as writable}
        reset(fd,1);
       end;
      inc(FtNurec);
      seek(fd,FtOff+(FtNurec-1)*FtRecl);
      FtAppend:=true
     end
    else
     seek(fd,FtOff+(recno-1)*FtRecl);
    Ft_OneRec[1]:=' ';
    k:=2;
    for i:=1 to FtNuField do
     if length(Ft_rec[i])<FtInfo[i].len then   {If the string was shortened,}
       begin
        for j:=1 to length(Ft_rec[i]) do
         begin
          Ft_OneRec[k]:=Ft_rec[i][j];
          inc(k);
         end;                  {rest of the field is filled with blanks.}
        for j:=length(Ft_rec[i])+1 to FtInfo[i].len do
         begin
          Ft_OneRec[k]:=' ';
          inc(k);
         end
       end
     else {the length is the same (or greater:extra characters are ignored)}
       for j:=1 to FtInfo[i].len do
        begin
         Ft_OneRec[k]:=Ft_rec[i][j];
         inc(k);
        end;
    BlockWrite(fd,Ft_OneRec,FtRecl);
  end
end;

Function FtFieldNum(F_Nam:string):byte;
                       {gives the number of the field aux; 0 when fails}
var aux: string[10];
begin
  if length(F_Nam)<10 then i:= length(F_Nam) else i:=10;
  aux:='';
  for j:=1 to i do aux:=aux+UpCase(F_Nam[j]);
  i:=0;
  for j:= 1 to FtNuField do
    if FtInfo[j].field = aux then i:=j;
  FtFieldNum:=i
end;

Procedure FtStruDescr(f:string);
           {creates a file describing structure of another FITS file}
begin
 FtNewField('FIELD_NAME','C',10,0);
 FtNewField('FIELD_TYPE','C',1,0);
 FtNewField('FIELD_LEN','N',3,0);
 FtNewField('FIELD_DEC','N',3,0);
 FtNewField('DESCRIPTIO','C',77,0);
 if FtCreate(f) then;
end;


Procedure FtDescField(fn:string;ft:char;flen,fdec:byte;fdesc:string);
           {adds a record to this file}
begin
 Ft_rec[1]:=fn;
 Ft_rec[2]:=ft;
 str(flen:3,Ft_rec[3]);
 str(fdec:3,Ft_rec[4]);
 Ft_rec[5]:=fdesc;
 if FtWrite(0,Ft_rec) then;
end;

procedure FtDisp(recno:longint); {displays the current record on the screen}
begin
writeln('The whole record No.',recno,' is');
for j:=1 to FtNuField do
 begin
  write(FtInfo[j].field,' ');
  if length(FtInfo[j].field)<FtInfo[j].len then
   for i:=1 to FtInfo[j].len-length(FtInfo[j].field) do
    write(' ')
 end;
writeln;
for j:=1 to FtNuField do
 begin
  write(Ft_rec[j],' ');
  if length(FtInfo[j].field)>FtInfo[j].len then
   for i:=1 to -1*(FtInfo[j].len-length(FtInfo[j].field)) do
    write(' ')
 end;
writeln;
end;


Procedure FtClose;
var Year,Month,Day,DoW:word; eofchr: byte;
begin
if FtOpen then
 if FtAppend then
  begin
   seek(fd,FtOff+(FtNurec)*FtRecl);
   eofchr:=26;
   BlockWrite(fd,eofchr,1);
   close(fd);
   assign(f1,FtFileName);
   reset(f1);
   read(f1,v1);
   GetDate(Year,Month,Day,DoW);
   v1.YY:=Year-1900;
   v1.MM:=Month;
   v1.DD:=Day;
   v1.reccount:=FtNuRec;
   seek(f1,0);
   write(f1,v1);
   close(f1)
  end
 else
  close(fd);
FileMode:=0; {will open the files by reset as Read Only}
FtNuField:=0;
FtOpen:=false;
end;

begin
FtOK:=true;
FtNuField:=0;
FtOpen:=False
end.
