 {~ Reads a record from the General Catalogue of Variable Stars (GCVSred.dbf)~}

unit gcvs; {gives information on a chosen GCVS star}
               {from a dBase file - J.Hollan, '94}
(*  Copyright (C) 1999 Jan Hollan;
  by "program" this unit is further meant.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)
interface
uses DbBridge, angles_o, str_num,
     Un_Str_M {instead of Unit_Str by Blaise Computing, Inc.};

function get_GCVS(var Desig,Con:string; var RA,Decl :real; var NoR:word):boolean;
 {V is designation, e.g. S, UW, BL, V1068, ALPHA, UUU, TAU_4
  and C the constellation short. Small letters allowed
  (but pay attention to Argelander / Bayer designation confusion )}

{needs text index file  G_Con_u.txt
 and the main data      GCVSred.dbf }
const
Loc_GCVS_const =
 'C:\d\gcvs;c:\d\j\d\stars\gcvs;c:\d\stars\gcvs;'+
 'g:\cats;f:\cats;'+
 '/home/jhollan/public_html/data;/home/guest/cats';
Loc_GCVS:string[110]=Loc_GCVS_const;
h_dir_gcvs=
 'LV<directory with the GCVS (Gen. Cat. of Variable Stars)>'+cl+
 '   (default '+Loc_GCVS_const+')>'+cl2;

implementation
var
 V : string[5];
 BV: array [0..6] of char absolute V;
 RV:string[4];
 C, Short_con: string[3];
 ind_line:string;
 OeR, OV, OA, OV2 : byte;
 rel, {ordinal number of star within the constellation: 0 for R, 1 for S,...}
 Start_con {starting record of the constellation within the dBase file},
 Max_Con {Maximal ordinal number within the constallation} : word;
 j: integer;
 last, {The stars designated by greek or Bayer letters are at the end of the
        constellation : becomes TRUE in this case}
 found : boolean;
 ind_fil : text;
 RA2MD, De2MD : longint;

function get_GCVS(var Desig,Con:string; var RA,Decl :real; var NoR:word):boolean;

begin
get_gcvs:=false;
C:=__cvtstr(Con,_to_upcase_str);
V:=Desig;
if pos('_',V)>0 then
 for j:=2 to length(V)-1 do
  if BV[j]='_' then BV[j]:=' ';
NoR:=0;

OeR:=ord('R');
OV:=ord(V[1]);
last:=False;
if length(V) = 1 then
{Bayer or one-letter Argelander designation}
 begin
  rel:=OV-OeR;
  if (rel<0) or (rel>10) then
{Bayer designation}
   begin
    last:=True;
    V:=__cvtstr(V,_to_upcase_str);
    {V:=V+V+V; - not needed now, after conversion from uppercase}
   end
 end
else
 begin
  V:=__cvtstr(V,_to_upcase_str);
  OV:=ord(V[1]);
  if ord(V[2])<58 then
{V#### type of designation}
   begin
    val(copy(v,2,length(v)-1),rel,j);
    dec(rel);
    if OV<>ord('V') then
     begin writeln(V,' is no var. star designation!'); exit end
   end
  else
{Argelander two-letter designation}
   if (length(V)=2)
   or ((length(V)>2) and (V[3]=' ')) then
    begin
     OV2:=ord(V[2]);
     if OV2<OV then
      begin
       writeln(
 'The second letter should not precede the first one alphabetically.');
       exit
      end;
     if OV>OeR-1 then
 {Second part of the Argelander's alphabete, starting with RR}
       rel:=9+ (((19+OeR-OV)*(OV-OeR)) div 2) +OV2-OV
     else
 {Third part, beginning with AA}
      begin
       OA:=ord('A');
       IF OV>ord('J') then OV:=OV-1;
                   {- That alphabete skips J from some curious reason! -}
       if OV2>ord('J') then OV2:=OV2-1;
       rel:=54 + (((51+OA-OV)*(OV-OA)) DIV 2) +OV2-OV
      end
    end
   else
    begin
     last:=True;
     if (V[1]=V[2]) and (V[2]=V[3]) then
      V:=V[1];
    end;
 end;

if last and (length(V)=1) then
 V:=V+' ';

if not rt_reset(Loc_gcvs,ind_fil,'g_con_u.txt') then
  begin
   writeln(
'The file G_Con_U.txt with the starting record numbers of the contained');
   writeln(
'constellations not found!');
     exit
  end;
repeat
 readln(ind_fil,ind_line);
 short_con:=__cvtstr(ItemStr(1,ind_line),_to_upcase_str);
 start_con:=I_S(ItemStr(2,ind_line));
until ((short_con=C) and (start_con>0)) or eof(ind_fil);

if not (short_con=C) then
 begin writeln('The constellation ',C, ' not found!'); exit end;

if last then
{Greek or Bayer designation - such stars should be within the ten last
 records of the corresponding constellation.}
 if eof(ind_fil) then
   start_con:=DbNuRec-10
 else
   begin
    readln(ind_fil,short_con,start_con);
    dec(start_con,11)
   end
else
 if eof(ind_fil) then
   Max_con:=DbNuRec+1-start_con
 else
   begin
    readln(ind_fil,short_con,Max_con);
    Max_Con:=Max_Con-Start_Con;
   end;


close(Ind_fil);

If not DbInitSearch(Loc_gcvs,'gcvsred.dbf') then
 begin
  writeln(
'The file with the variable stars catalog GCVSred.dbf not found!');
  exit
 end;


if not last then
 begin
  repeat
   if rel>=Max_Con then
    begin
     writeln('The variable stars in the constellation are not named that far.');
     exit
    end;
   if not DbRead(Start_con + rel,rec) then
    begin
     writeln('Reaching behind the end of GCVSred.dbf. Wrong g_con_u.txt?');
     exit
    end;
   inc(rel);
  until copy(rec[DbFieldNum('DESIG')],1,length(V))=V;
  NoR:=Start_con+rel-1
 end
else
 begin
  rel:=0;
  repeat
   if not DbRead(Start_con + rel,rec) then exit;
   ind_line:=rec[DbFieldNum('DESIG')];
   if (ind_line[1]=ind_line[2]) and (ind_line[3]=ind_line[2]) then
    ind_line:=ind_line[1]+copy(ind_line,4,length(ind_line)-3);
   found:=__cvtstr(copy(ind_line,1,length(V)),_to_upcase_str)
          =V;
   inc(rel);
  until found or (rel=11);
  if (rel=11) and not found then
    begin
     if length(V)>1 then
{skipping the last letter - e.g., to allow ALPH2 match with ALPHA}
      begin
       RV:= copy(V,1,length(V)-1);
       rel:=0;
       repeat
        if not DbRead(Start_con + rel,rec) then exit;
        found:=copy(rec[DbFieldNum('DESIG')],1,length(RV))=RV;
        inc(rel);
       until found or (rel=11)
      end;
     if (rel=11) and not found then
      begin
       writeln(
'The star ',V,' not found within the constellation ',con,'!'); exit end
     else NoR:=Start_con + rel
    end
  else NoR:=Start_con + rel
 end;

{DbClose;}

val(__CvtStr(rec[DbFieldNum('ALPHA')],_rem_lead_white_str),RA2mD,j);
RA:=RA2mD/1000;
if j>0 then
{lower precision}
 begin
  val(__CvtStr(rec[DbFieldNum('ALPHA')],_rem_white_str),RA2mD,j);
  RA:=RA2mD/10
 end;
val(__CvtStr(rec[DbFieldNum('DELTA')],_rem_lead_white_str),De2mD,j);
Decl:=De2mD/1000;
if j>0 then
 begin
  val(__CvtStr(rec[DbFieldNum('DELTA')],_rem_white_str),De2mD,j);
  Decl:=De2mD/100
 end;
desig:=rec[DbFieldNum('DESIG')];
con:=rec[DbFieldNum('CON')];

get_gcvs:=true;
end;

end.
