 {~ Ephemeris (helio/geo-centric) of 10 "planets" and Sun-orbiting bodies ~}

UNIT PLANETs;
(*  Copyright (C) 1999 Jan Hollan;
  by "program" this unit is further meant.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)
 {  According to the book
      Pokorny, Z.: Astronomicke algoritmy pro kalkulatory (in Czech,
                   mostly applying Astronomical formulae by J.Meeus),
                   Hvezdarna a planetarium hl. m. Prahy, Praha 1988

      written basically by David Motl, Brno 1993.
    Procedure Distances of Planets (Dist_of_Plan) was written by R.K.Novak.
    Adaptation  RA/h -> RA/degree, Distance of Moon and present commentary
    by Jan Hollan.
  }


INTERFACE
uses solar_ut, angles_o, str_num, precesse;

{Input  JD :     full Julian date of the epoch
          (and equinox of the coordinates, neglecting nutation).
 All JDs should be in Temps Terrestrique
 (Ephemeride Time, as calld earlier) with exception of the
 subterr_lat_posi_angD procedure -- its input is to be
 UTC(i.e., rotational)-based JD, as it adds the TT-UTC correction itself.

 Output l, b : ecliptical coordinates in angular degrees }

{ N_of_planet is mostly 10 for Sun and 11 for Moon,
 whereas in the programmes using the unit Sun is 0 and Moon 10 !}

Procedure Planet_Hel
 (N_of_planet:byte;jd:real; var l,b,r:real; var Name:string);
Procedure Hel_to_Geo(jd,l,b,r:real;   {heloicentric }
                    var lg,bg,rg:real {geocentric ecliptical coordinates} );
function Planet_faintness(N_of_planet:byte;ph_a,rs,rg:real):real;

procedure subterr_lat_posi_angD(N_of_planet:byte; ra_pd, de_pd,
                                ra_EarthNPd, decl_EarthNPd:real;
                                var pa, lat_Earth: real);
function subterr_longD(N_of_planet,syst:byte;
                       JD_mom, ra_pd, de_pd, r_geoc:real):real;
Procedure Moon_Geo (jd:real;
                var lm, bm,
                dm {/ 1 Earth radius}, fam {phase, crudely} :real); {Moon}
procedure object_hel(a,q,exc,i,AN,PN,t0,t: real;
                     var xe,ye,ze,r {/ 1 AU} :real);
function Heliocentric_Correction_s(jdat,ra_l,de_b:real):real;
              {coordinates are assumed J2000.0, correction is some
                minus five hundred seconds
                  when the fixed star is  behind the Sun, and
                plus five hundred seconds
                  when it is in the opposite direction}
procedure Init_Moon_pole(JD_p:real);

const
 Planet_Names:array[0..10] of string[7]=
(
'Sun    ',
'Mercury',
'Venus  ',
'Earth  ',
'Mars   ',
'Jupiter',
'Saturn ',
'Uranus ',
'Neptune',
'Pluto  ',
'Moon   ');
 planet_pole: array [1..11,1..4] of real=(
{RA(Npole)decl.      incl. asc.node (from the ecliptics) , all J2000.0)}
(281.01, 61.45    , 7.00 , 48.33), {Mercury}
(272.69, 67.17    , 3.39 , 76.68), {Venus}
(  0   , 90       , 0    ,   0  ), {Earth}
(317.69, 52.89    , 1.85 , 49.56), {Mars}
(268.05, 64.49    , 1.30 ,100.45), {Jupiter}
( 40.57, 83.53    , 2.49 ,113.66), {Saturn}
(257.43,-15.10    , 0.77 , 73.98), {Uran}
(298.52, 40.75    , 1.77 ,131.78), {Neptune}
(311.52,  4.17    ,17.1  ,110.3 ), {Pluto}
(286.12, 63.87    , 7.25 , 75.76), {Sun, both pairs concern rotation }
(270   , 66.56    ,1.54242,  0   ) {Moon       -      "       -      ,
   longitude changes periodically, given equatorial coordinates are mean ones
   (at the pole of ecliptics), actual values are to be set by Init_moon_pole }
);

var planet_ver_eq:array[1..11] of real;

IMPLEMENTATION

var

 {All these variables are needed by some other procedures than the source one.
  Some of them influence Hel_Geo (through Earth position),
  t, vt, vLS and vL2 are used by Moon_Geo.
  All these variables are given values by Init.
  }
    t,vt:real;
    vL2,m2,u2:real;    {Venus}
        m4   :real;    {Mars}
        m5   :real;    {Jupiter}
        m6   :real;    {Saturn}
    vL7,m7   :real;    {Uran}
        m8   :real;    {Neptune}
    vLs,ms   :real;    {Sun}
    vLm      :real;    {Moon}

Procedure Hel_to_Geo(jd,l,b,r:real; var lg,bg,rg:real);
var lE,rE,xE,yE,x,y,z:real; Name:string;
begin
 Planet_Hel(3,jd,lE,bg,rE,Name);
 P3d_to_C(r,l,b,x,y,z);
 P2d_to_C(rE,lE,xE,yE);
 C3_to_Pd(x-xE,y-yE,z,rg,lg,bg);
end;

function Planet_faintness(N_of_planet:byte;ph_a,rs,rg:real):real;
var x:real; j:byte;
const
 pl_f_c:array[0..11,0..4] of real=(
(0.0, 0.038, -0.000273, 0.000002, 0), {body 0 mag, phase dependent as Mercury}
(-0.42, 0.038, -0.000273, 0.000002, 0), {Mercury}
(-4.4, 0.0009, 0.000239, -0.00000065, 0), {Venus}
( -3.86,0,0,0,0),    {Earth:nonsense}
(-1.52,0.01486, 0, 0, 0),  {Mars}
(-9.4,0,0,0,0),
(-8.88,0.044,0,0,0),
(-7.19,0,0,0,0),
(-6.87,0,0,0,0),
(-1.01,0,0,0,0),
(-26.8,0,0,0,0),
(0.23,0.026,0,0,4.0E-9)
{(0.21, 0.038, -0.000273, 0.000002) 
 Moon phase dependent as Mercury before Oct 2001}
);

{    Mercury:   -0.36 + 5*log10(r*R) + 0.027 * FV + 2.2E-13 * FV**6
    Venus:     -4.34 + 5*log10(r*R) + 0.013 * FV + 4.2E-7  * FV**3
    Mars:      -1.51 + 5*log10(r*R) + 0.016 * FV
    Jupiter:   -9.25 + 5*log10(r*R) + 0.014 * FV
    Saturn:    -9.0  + 5*log10(r*R) + 0.044 * FV + ring_magn
    Uranus:    -7.15 + 5*log10(r*R) + 0.001 * FV
    Neptune:   -6.90 + 5*log10(r*R) + 0.001 * FV

    Moon:      +0.23 + 5*log10(r*R) + 0.026 * FV + 4.0E-9 * FV**4
 acc. to    
   Linkname: Computing planetary positions
        URL: http://hotel04.ausys.se/pausch/comp/ppcomp.html#15
}

begin
 if N_of_Planet<>10 then
  begin
   x:=pl_f_c[N_of_planet,4];
   for j:=1 to 4 do
    x:=ph_a*x+pl_f_c[N_of_planet,4-j];
   Planet_faintness:=5*ln(rs*rg)/l10+x;
  end
 else       {Sun}
  Planet_faintness:=5*ln(   rg)/l10+pl_f_c[10,0];
end;

procedure subterr_lat_posi_angD(N_of_planet:byte; ra_pd, de_pd,
                                ra_EarthNPd, decl_EarthNPd:real;
                                var pa, lat_Earth: real);
var ra_plpd, de_plpd:real;
begin
 if N_of_planet in [1..11] then
  begin
   ra_plpd:=planet_pole[N_of_planet,1]; de_plpd:=planet_pole[N_of_planet,2];
   ang_dist_posi_angD2(ra_pd,de_pd,ra_EarthNPd,decl_EarthNPd,ra_plpd,de_plpd,
                       pa,lat_Earth);
  {This was in fact transformation to a system with N pole in [ra_pd, de_pd]}
   if pa>180 then pa:=pa-360;
   lat_Earth:=Lat_Earth-90;
  end
 else
  begin pa:=0; lat_Earth:=0; end;
end;

function subterr_longD(N_of_planet,syst:byte;
                       JD_mom, ra_pd, de_pd, r_geoc:real):real;
const planet_rot:array[1..5,1..7] of real = (
(148.368, 350.891986, 2433282.5, 0.000096, 2.0266, 0, 1),
(281.001, 877.90,     2414120.0, 0.000076, 5.07040,0, 1),
(330.002, 870.27,     2414120.0, 0.000076, 5.02633,0, 1),
(      0, 14.18439716,2398220.0, 0       , 0.08186,0,-1),
(      0, 13.1763585 ,   JD2000, 0       , 0      ,-38.2,-1));
                                                    {this constant
                 has been chosen to match accurate lunar ephemeris
       like those in Astronomical Almanac or those by JanVondr\'ak)}
 var ra_00,de_00,sub_00:real; i:integer;
begin
 if N_of_planet in [4,5,10,11] then
  begin
   case N_of_planet of
    4: i:=1;
    5: i:=1+syst;
   10,11: i:=N_of_planet-6;
   end;
     {planetographic longitude of its own vernal equinox:}
   sub_00:=Bas_Int(planet_rot[i,1]+planet_rot[i,2]
       *(jd_mom-planet_rot[i,3]+TT_UTC_in_days(JD_mom)-planet_rot[i,4]),360);
     {RA' of the Earth, counted from the point, where the planet's
      equator ascends above the Earth equator:}
   if N_of_planet=11 then
    begin
     Init_moon_pole(JD_mom);
     ra_00:=deg_to_rad(270-planet_pole[N_of_planet,1]);
     de_00:=0;
     ra_d2l_b(deg_to_rad(90-planet_pole[N_of_planet,2]),ra_00,de_00);
     planet_ver_eq[N_of_planet]:=rad_to_deg(ra_00);
      {but this concerns Earth's vernal equinox J2000.0, not the Moon's one}
    end;
   ra_pd:=deg_to_rad(ra_pd+180+270-planet_pole[N_of_planet,1]);
     {Declination of the Earth:}
   de_pd:=deg_to_rad(-de_pd);
     {rotating around the nodal line to get Earth position in some kind
      of planet's equatorial coordinates
      (nodal lines conects the intersections of equators,
      rotation is like equatorial->ecliptical coord. conversion) }
   ra_d2l_b(deg_to_rad(90-planet_pole[N_of_planet,2]),ra_pd,de_pd);
   subterr_longD:=Bas_int
           (planet_rot[i,7]*
             (sub_00
              -rad_to_deg(ra_pd)+planet_ver_eq[N_of_planet]
              -planet_rot[i,6]     {forced shift, should be zero}
              -planet_rot[i,5]*r_geoc  {light-time delay of visible position}
              )
            ,360);
  end
 else
  subterr_longD:=0;
end;



Procedure Init(jd:real);
begin
 t:=jd-2451545;
 vt:=1+(t/36525);
    {further, all v?? are in angular seconds,
     whereas m? and u? are in radians}
        {Mesic}
  vLm:=Bas_Int((2183.1624 +131.76396464   *t){revolutions *3600}
              ,3600 {to get a fraction of revolution * 3600} )
      *360;         {to get angle / 1 arcsecond}
        {Venus}
 vL2:=Bas_Int((1819.7928  +16.021687212  *t),3600)*360;
 M2:= (   0.140023+ 0.00445036173*t)*2*pi;
 u2:= (   0.292498+ 0.00445040017*t)*2*pi;
        {Mars}
 M4:= (   0.053856+ 0.00145561327*t)*2*pi;
        {Jupiter}
 M5:= (   0.056531+ 0.00023080893*t)*2*pi;
        {Saturn}
 M6:= (   0.882987+ 0.00009294371*t)*2*pi;
        {Uranus}
 vL7:=Bas_Int((3132.6084  + 0.117699768  *t),3600)*360;
 M7:= (   0.400589+ 0.00003269438*t)*2*pi;
        {Neptune}
 M8:= (   0.725368+ 0.00001672092*t)*2*pi;
         {Sun}
 vLs:=Bas_Int((2804.6592  + 9.856473516  *t),3600)*360;
 ms:=(    0.993126+ 0.00273777850*t)*2*pi;
end;

Procedure Moon_Geo(jd:real; var lm,bm,dm,fam:real);
var Ls,L2,mm,um,fm,om:real;
begin
 Init(JD);
 mm:=(    0.374897+ 0.03629164709*t)*2*pi;
 um:=(    0.259091+ 0.03674819520*t)*2*pi;
 fm:=(    0.827362+ 0.03386319198*t)*2*pi;
 om:=(    0.347343- 0.00014709391*t)*2*pi;
 Ls:=16*deg_to_rad(vLs/3600);
 L2:=18*deg_to_rad(vL2/3600);
 lm:=vlm+22640*sin(mm)-4586*sin(mm-2*fm)+2370*sin(2*fm)+769*sin(2*mm)-668*sin(ms)
  -412*sin(2*um)-212*sin(2*mm-2*fm)-206*sin(mm-2*fm+ms)+192*sin(mm+2*fm)
  +165*sin(2*fm-ms)+148*sin(mm-ms)-125*sin(fm)-110*sin(mm+ms)-55*sin(2*um-2*fm)
  -45*sin(mm+2*um)+40*sin(mm-2*um)-38*sin(mm-4*fm)+36*sin(3*mm)-31*sin(2*mm-4*fm)
  +28*sin(mm-2*fm-ms)-24*sin(2*fm+ms)+19*sin(mm-fm)+18*sin(fm+ms)+15*sin(mm+2*fm-ms)
  +14*sin(2*mm+2*fm)+14*sin(4*fm)-13*sin(3*mm-2*fm)-11*sin(mm-Ls-L2)
  +10*sin(2*mm-ms)+9*sin(mm-2*um-2*fm)+9*cos(mm+Ls-L2)
  -9*sin(2*mm-2*fm+ms)-8*sin(mm-fm)+8*sin(2*fm-2*ms)-8*sin(2*ms+ms)-7*sin(2*ms)
  -7*sin(mm-2*fm+2*ms)+7*sin(om)-6*sin(mm-2*um+2*fm)-6*sin(2*um+2*fm)-4*sin(mm-4*fm+ms)
  +4*vt*cos(mm+Ls-L2)-4*sin(2*mm-2*um)+4*vt*sin(mm+Ls-L2)
  +3*sin(mm-3*fm)-3*sin(mm+2*fm+ms)-3*sin(2*mm-4*fm+ms)-3*sin(mm-2*ms)
  +3*sin(mm-2*fm-2*ms)-2*sin(2*mm-2*fm-ms)-2*sin(2*um-2*fm+ms)+2*sin(mm+4*fm)
  +2*sin(4*mm)+2*sin(4*fm-ms)+2*sin(2*mm-fm);
 bm:=18461*sin(um)+1010*sin(mm+um)+1000*sin(mm-um)-624*sin(um-2*fm)
  -199*sin(mm-um-2*fm)-167*sin(mm+um-2*fm)+117*sin(um-2*fm)+62*sin(2*mm+um)
  +33*sin(mm-um+2*fm)+32*sin(2*mm-um)+30*sin(um-2*fm+ms)-16*sin(2*mm+um-2*fm)
  +15*sin(mm+um+2*fm)+12*sin(um-2*fm-ms)-9*sin(mm-um-2*fm+ms)-8*sin(um-om)
  +8*sin(um+2*fm-ms)-7*sin(mm+um-2*fm+ms)+7*sin(mm+um-ms)-7*sin(mm+um-4*fm)
  -6*sin(um+ms)-6*sin(3*um)+6*sin(mm-um-ms)-5*sin(um-fm)-5*sin(mm+um+ms)
  -5*sin(mm-um+ms)+5*sin(um-ms)+5*sin(um-fm)+4*sin(3*mm+um)-4*sin(um-4*fm)
  -3*sin(mm-um-4*fm)+3*sin(mm-3*um)-2*sin(2*mm-um-4*fm)-2*sin(3*um-2*fm)
  +2*sin(2*mm-um+2*fm)+2*sin(mm-um+2*fm-ms)+2*sin(2*mm-um-2*fm)+2*sin(3*mm-um);
 dm:=60.36298
  -3.27746*cos(mm)       +0.00398*cos(mm-2*fm-ms)
  -0.57994*cos(mm-2*fm)  -0.00366*cos(3*mm)
  -0.46357*cos(2*fm)     -0.00295*cos(2*mm-4*fm)
  -0.08904*cos(2*mm)     -0.00263*cos(fm+ms)
  +0.03865*cos(2*mm-2*fm)+0.00249*cos(3*mm-2*fm)
  -0.03237*cos(2*fm-ms)  -0.00221*cos(mm+2*fm-ms)
  -0.02688*cos(mm+2*fm)  +0.00185*cos(2*um-2*fm)
  -0.02358*cos(mm-2*fm+ms)
  -0.02030*cos(mm-ms)    -0.00161*cos(2*fm-2*ms)
  +0.01719*cos(fm)       +0.00147*cos(mm+2*um-2*fm)
  +0.01671*cos(mm+ms)    -0.00142*cos(4*fm)
  +0.01247*cos(mm-2*um)  +0.00139*cos(2*mm-2*fm+ms)
  +0.00704*cos(ms)       -0.00118*cos(mm-4*fm+ms)
  +0.00529*cos(2*fm+ms)  -0.00116*cos(2*mm-2*fm)
  -0.00524*cos(mm-4*fm)  -0.00110*cos(2*mm-ms);
 fam:=frac(frac((jd-1721088.5)/29.53059)+0.25);
 lm:=Bas_Int(lm/3600,360); bm:=bm/3600;
end;



Procedure Planet_Hel
 (N_of_planet:byte;jd:real; var l,b,r:real; var Name:string);

 Procedure Earth;
  begin
   l:=vLs+6910*sin(ms)+72*sin(2*ms)-17*vt*sin(ms)-7*cos(ms-m5)+6*sin((vlm-vls)/324000*pi)
    +5*sin(4*ms+8*m4+3*m5)-5*cos(2*ms-2*m2)-4*sin(ms-m2)+4*cos(4*ms-8*m4+3*m5)
    +3*sin(2*ms-2*m2)-3*sin(m5)-3*sin(2*ms-2*m5)
    +648000 {=180*3600};
   b:=0;
   r:=1.00014-0.01675*cos(ms)-0.00014*cos(2*ms);
   Name:='Earth  ';
  end;

 Procedure Mercury;
  var vL1,m1,u1:real;
  begin
   vL1:=Bas_Int((2522.502   +40.9237704    *t),3600)*360;
   m1:= (   0.485541+ 0.01136759566*t)*2*pi;
   u1:= (   0.566441+ 0.01136762384*t)*2*pi;
   l:=vl1+(84378*sin(m1))+(10733*sin(2*m1))+(1892*sin(3*m1))-(646*sin(2*u1))
    +(381*sin(4*m1))-(306*sin(m1-2*u1))-(274*sin(m1+2*u1))
    -(92*sin(2*m1+2*u1))+(83*sin(5*m1))-(28*sin(3*m1+2*u1))+(25*sin(2*m1-2*u1))
    +(19*sin(6*m1))-(9*sin((4*m1)+2*u1))+(8*vt*sin(m1))+(7*cos(m1-5*m2));
   b:=(24134*sin(u1))+(5180*sin(m1-u1))+(4910*sin(m1+u1))+(1124*sin(2*m1+u1))
    +(271*sin(3*m1+u1))+(132*sin(2*m1-u1))+(67*sin(4*m1+u1))+(18*sin(5*m1+u1))
    -(10*sin(3*u1))+(9*sin(m1-3*u1));
   r:=0.39528-0.07834*cos(m1)-0.00795*cos(2*m1)-0.00121*cos(3*m1)-0.00022*cos(4*m1);
   Name:='Mercury';
  end;

 Procedure Venus;
  begin
   l:=vL2+2814*sin(m2)-181*sin(2*u2)-20*vt*sin(m2)+12*sin(2*m2)-10*cos(2*ms-2*m2)+7*cos(3*ms-3*m2);
   b:=12215*sin(u2)+83*sin(m2+u2)+83*sin(m2-u2);
   r:=0.72335-0.00493*cos(m2);
   Name:='Venus  ';
  end;

 Procedure Mars;
   var vL4,u4:real;
  begin
   vL4:=Bas_Int((3554.4708  + 5.240711808  *t),3600)*360;
   u4:= (   0.849694+ 0.00145569465*t)*2*pi;
   l:=vl4+38451*sin(m4)+2238*sin(2*m4)+181*sin(3*m4)-52*sin(2*u4)+37*vt*sin(m4)
    -22*cos(m4-2*m5)-19*sin(m4-m5)+17*cos(m4-m5)+17*sin(4*m4)-16*cos(2*m4-2*m5)
    +13*cos(ms-2*m4)-10*sin(m4-2*u2)-10*sin(m4+2*u4)+7*cos(ms-m4)-7*cos(2*ms-3*m4)
    -5*sin(m2-3*m4)-5*sin(ms-m4)-5*sin(ms-2*m4)-4*cos(2*ms-4*m4)+4*vt*sin(2*m4)
    +4*cos(m5)+3*cos(m2-3*m4)+3*cos(2*m4-2*m5);
   b:=6603*sin(u4)+622*sin(m4-u4)+615*sin(m4+u4)+64*sin(2*m4+u4);
   r:=1.53031-0.14170*cos(m4)-0.00660*cos(2*m4)-0.00047*cos(3*m4);
   Name:='Mars   ';
  end;

 Procedure Jupiter;
  var vL5:real;
  begin
   vL5:=Bas_Int(( 322.5888  + 0.830912148  *t),3600)*360;
   l:=vl5+19934*sin(m5)+5023*vt+2511+1093*cos(2*m5-5*m6)+601*sin(2*m5)
    -479*sin(2*m5-5*m6)-185*sin(2*m5-2*m6)+137*sin(3*m5-5*m6)-131*sin(m5-2*m6)
    +79*cos(m5-m6)-76*cos(2*m5-2*m6)-74*vt*cos(m5)+68*vt*sin(m5)+66*cos(2*m5-3*m6)
    +63*cos(3*m5-5*m6)+53*cos(m5-5*m6)+49*sin(2*m5-3*m6)-43*vt*sin(2*m5-5*m6)
    -37*cos(m5)+25*sin(vl5/324000*pi)+25*sin(3*m5)-23*sin(m5-5*m6)-19*vt*sin(2*m5-5*m6)
    +17*cos(2*m5-4*m6)+17*cos(3*m5-3*m6)-14*sin(m5-m6)-13*sin(3*m5-4*m6)
    -9*cos(vl5/648000*pi)+9*cos(m6)-9*sin(m6)-9*sin(3*m5-2*m6)+9*sin(4*m5-5*m6)
    +9*sin(2*m5-6*m6+3*m7)-8*cos(4*m5-10*m6)+7*cos(3*m5-4*m6)-7*cos(m5-3*m6)
    -7*sin(4*m5-10*m6)-7*sin(m5-3*m6)+6*cos(4*m5-5*m6)-6*sin(3*m5-3*m6)+5*cos(2*m6)
    -4*sin(4*m5-4*m6)-4*cos(3*m6)+4*cos(2*m5-m6)-4*cos(3*m5-2*m6)-4*vt*cos(2*m5)
    +3*vt*sin(2*m5)+3*cos(5*m6)+3*cos(5*m5-10*m6)+3*sin(2*m6)-2*sin(vl5/324000*pi-m5)
    +2*sin(vl5/324000*pi+m5)-2*vt*sin(3*m5-5*m6)-2*vt*sin(m5-5*m6);
   b:=-4692*cos(m5)+259*sin(m5)+227-227*cos(2*m5)+30*vt*sin(m5)+21*vt*cos(m5)
    +16*sin(3*m5-5*m6)-13*sin(m5-5*m6)-12*cos(3*m5)+12*sin(2*m5)+7*cos(3*m5-5*m6)
    -5*cos(m5-5*m6);
   r:=5.20883-0.25122*cos(m5)-0.00604*cos(2*m5)+0.00260*cos(2*m5-2*m6)
    -0.00170*cos(3*m5-5*m6)-0.00091*vt*sin(m5)-0.00084*vt*cos(m5)+0.00069*sin(2*m5-3*m6)
    -0.00067*sin(m5-5*m6)+0.00066*sin(3*m5-5*m6)+0.00063*sin(m5-m6)
    -0.00051*cos(2*m5-3*m6)-0.00046*sin(m5)-0.00029*cos(m5-5*m6)+0.00027*cos(m5-2*m6)
    -0.00022*cos(3*m5)-0.00021*sin(2*m5-5*m6);
   Name:='Jupiter';
  end;

 Procedure Saturn;
   var vL6:real;
  begin
   vL6:=Bas_Int(( 479.862   + 0.334597356  *t),3600)*360;
   l:=vl6+23045*sin(m6)+5014*vt-2689*cos(2*m5-5*m6)+2507+1177*sin(2*m5-5*m6)
    -826*cos(2*m5-4*m6)+802*sin(2*m6)+425*sin(m5-2*m6)-229*vt*cos(m6)
    -153*cos(2*m5-6*m6)-142*vt*vt*sin(m6)-114*cos(m6)+101*vt*sin(2*m5-5*m6)
    -70*cos(vl6/324000*pi)+67*sin(vl6/324000*pi)+66*sin(2*m5-6*m6)+60*vt*cos(2*m5-5*m6)
    +41*sin(m5-3*m6)+39*sin(3*m6)+31*sin(m5-m6)+31*sin(2*m5-2*m6)-29*cos(2*m5-3*m6)
    -28*sin(2*m5-6*m6+3*m7)+28*cos(m5-3*m6)+22*vt*sin(2*m5-4*m6)-22*sin(m6-3*m7)
    +20*sin(2*m5-3*m6)+20*cos(4*m5-10*m6)+19*cos(2*m6-3*m7)+19*sin(4*m5-10*m6)
    -17*vt*cos(2*m6)-16*cos(m6-3*m7)-12*sin(2*m5-4*m6)+12*cos(m5)-12*sin(2*m6-2*m7)
    -11*vt*sin(2*m6)-11*cos(2*m5-7*m6)+10*sin(2*m6-3*m7)+10*cos(2*m5+2*m6)
    +9*sin(4*m5-9*m6)-8*sin(m6-2*m7)-8*cos(vl6/324000*pi+m6)+8*cos(vl6/324000*pi-m6)
    +8*sin(m6-m7)-8*sin(vl6/324000*pi-m6)+7*sin(vl6/324000*pi+m6)-7*cos(m5-2*m6)
    -7*cos(2*m6)-6*vt*sin(4*m5-10*m6)+6*vt*cos(4*m5-10*m6)+6*vt*sin(2*m5-6*m6)
    -5*sin(3*m5-7*m6)-5*cos(3*m5-7*m6)-5*cos(2*m6-2*m7)+5*sin(3*m5-4*m6)
    +5*sin(2*m5-7*m6)+4*sin(3*m5-5*m6)+4*vt*cos(m5-2*m6)+3*vt*cos(3*m5-4*m6)
    +3*cos(2*m5-4*m6)+3*cos(2*m5-6*m6+3*m7)-3*vt*sin(vl6/324000*pi)+3*vt*cos(2*m5-6*m6)
    -3*vt*cos(vl6/324000*pi)+3*cos(3*m5-7*m6)+3*cos(4*m5-9*m6)+3*sin(3*m5-6*m6)
    +3*sin(2*m5-m6)+3*sin(m5-4*m6)+2*cos(3*m6-3*m7)+2*vt*sin(m5-2*m6)+2*sin(4*m6)
    -2*cos(3*m5-4*m6)-2*cos(2*m5-m6)-2*sin(2*m5-7*m6+3*m7)+2*cos(m5-4*m6)
    +2*cos(4*m5-11*m6)-2*sin(m6-m7);
   b:=8297*sin(m6)-3346*cos(m6)+462*sin(2*m6)-189*cos(2*m6)+185-79*vt*cos(m6)
    -71*cos(2*m5-4*m6)+46*sin(2*m5-4*m6)-45*cos(2*m5-6*m6)-29*sin(3*m6)
    -14*cos(2*m5-5*m6)-11*cos(3*m6)-10*vt+9*sin(m5-3*m6)+8*sin(m5-m6)-6*sin(2*m5-3*m6)
    +5*sin(2*m5-7*m6)-5*cos(2*m5-7*m6)+4*sin(2*m5-5*m6)-4*vt*sin(2*m6)-3*cos(m5-m6)
    +3*cos(m5-3*m6)+3*vt*sin(2*m5-4*m6)+3*sin(m5-2*m6)+2*sin(4*m6)-2*cos(2*m5-2*m6);
   r:=9.55774-0.53252*cos(m6)-0.01878*sin(2*m5-4*m6)-0.01482*cos(2*m6)
    +0.00817*sin(m5-m6)-0.00539*cos(m5-2*m6)-0.00524*vt*sin(m6)+0.00349*sin(2*m5-5*m6)
    +0.00328*vt*cos(m6)-0.00225*sin(m6)+0.00149*cos(2*m5-6*m6)-0.00126*cos(2*m5-2*m6)
    +0.00104*cos(m5-m6)+0.00101*cos(2*m5-5*m6)+0.00098*cos(m5-3*m6)
    -0.00073*cos(2*m5)-0.00062*cos(3*m6)+0.00042*sin(2*m6-3*m7)
    +0.00041*sin(2*m5-2*m6)-0.0004*sin(m5-3*m6)+0.0004*cos(2*m5-4*m6)-0.00028*vt
    -0.00023*sin(m5)+0.0002*sin(2*m5-7*m6);
   Name:='Saturn ';
  end;

 Procedure Uranus;
   var u7:real;
  begin
   u7:= (   0.664614+ 0.00003265562*t)*2*pi;
   l:=vl7+19397*sin(m7)+570*sin(2*m7)-536*vt*cos(m7)+143*sin(m6-2*m7)
    +110*vt*sin(m7)+102*sin(m6-3*m7)+76*cos(m6-3*m7)-49*sin(m5-m7)+32*vt*vt
    -30*vt*cos(2*m7)+29*sin(2*m5-6*m6+3*m7)+29*cos(2*m7-2*m8)-28*cos(m7-m8)
    +23*sin(3*m7)-21*cos(m5-m7)+20*sin(m7-m8)+20*cos(m6-2*m7)-19*cos(m6-m7)
    +17*sin(2*m7-3*m8)+14*sin(3*m7-3*m8)+13*sin(m6-m7)-12*vt*vt*cos(m7)-12*cos(m7)
    +10*sin(2*m7-2*m8)-9*sin(2*u7)-9*vt*vt*sin(m7)+9*cos(2*m7-3*m8)+8*vt*cos(m6-2*m7)
    +7*vt*cos(m6-2*m7)-7*vt*sin(2*m7)+7*vt*sin(2*m7)+6*sin(2*m5-6*m6+2*m7)
    +6*cos(2*m5-6*m6+2*m7)+5*sin(m6-4*m7)-4*sin(3*m7-4*m8)+4*cos(3*m7-3*m8)-3*cos(m8)
    -2*sin(m8);
   b:=2775*sin(u7)+131*sin(m7-u7)+130*sin(m7-u7);
   r:=19.21216-0.90154*cos(m7)-0.02488*vt*sin(m7)-0.00585*cos(m6-2*m7)
    -0.00508*vt*cos(m7)-0.00451*cos(m5-m7)+0.00336*sin(m6-m7)+0.00198*sin(m5-m7)
    +0.00118*cos(m6-3*m7)+0.00107*sin(m6-2*m7)-0.00103*vt*sin(2*m7)-0.00081*cos(3*m7-3*m8);
   Name:='Uranus ';
  end;

 Procedure Neptune;
  var vL8,u8:real;
  begin
   vL8:=Bas_Int((3048.8832  + 0.060195312  *t),3600)*360;
   u8:= (   0.480856+ 0.00001663715*t)*2*pi;
   l:=vl8+3523*sin(m8)-50*sin(2*u8)-43*vt*cos(m8)+29*sin(m5-m8)+19*sin(2*m8)
    +19*sin(2*m8)-18*cos(m5-m8)+13*cos(m6-m8)+13*sin(m6+m8)-9*sin(2*m7-3*m8)
    +9*cos(2*m7-2*m8)-5*cos(2*m7-3*m8)-4*vt*sin(m8)+4*cos(m7-2*m8)+4*vt*vt*sin(m8);
   b:=6404*sin(u8)+55*sin(m8+u8)+55*sin(m8-u8)-33*vt*sin(u8);
   r:=30.07175-0.25701*cos(m8)-0.00797*cos((vl7-vl8)/324000*pi-m7)
    +0.00409*cos(m5-m8)-0.00314*vt*sin(m8)+0.00250*sin(m5-m8)-0.00194*sin(m6-m8)
    +0.00185*cos(m6-m8);
   Name:='Neptune';
  end;

 Procedure Pluto;
  var vL9,m9,u9:real;
  begin
   vL9:=Bas_Int((2389.8744  + 0.040157352  *t),3600)*360;
   M9:= (   0.041020+ 0.00001104864*t)*2*pi;
   u9:= (   0.357355+ 0.00001104864*t)*2*pi;
   l:=vl9+101577*sin(m9)+15517*sin(2*m9)-3593*sin(2*u9)+3414*sin(m9-2*u9)
    -1871*sin(m9-2*u9)+839*sin(4*m9)-757*sin(2*m9+2*u9)-285*sin(3*m9+2*u9)
    +227*vt*vt*sin(m9)+218*sin(2*m9-2*u9)+200*vt*sin(m9);
   b:=57726*sin(u9)+15257*sin(m9-u9)+14102*sin(m9+u9)+3870*sin(2*m9+u9)
    +1138*sin(3*m9+u9)+472*sin(2*m9-u9)+353*sin(4*m9+u9)-144*sin(m9-3*u9)
    -119*sin(3*u9)-111*sin(m9+3*u9);
   r:=40.74638-9.58235*cos(m9)-1.16703*cos(2*m9)-0.22649*cos(3*m9)-0.04996*cos(4*m9);
   Name:='Pluto  ';
  end;

 begin {of the body of Planet_Hel}
  Init(jd);
  case N_Of_Planet of
   1: Mercury;
   2: Venus;
   3: Earth;
   4: Mars;
   5: Jupiter;
   6: Saturn;
   7: Uranus;
   8: Neptune;
   9: Pluto;
  end;
  l:=Bas_Int(l/3600,360); b:=b/3600;
 end; {of Planet_Hel}

{-----------------------------------------------------------------------
Heliocentric cartesian ecliptical co-ordinates of an object, whose
orbital elements are
   a:  semiaxis_large / 1 AU       (when 0, then q should be given)
   q:  perihelion  distance / 1 AU (when 0, then a should be given)
 exc:  numerical excentricity
   i:  inclination of the oriented orbital plane to the ecliptics
  AN:  longitude of the ascending node
  PN:  perihelion elongation from the ascending node
  t0   time of perihelion passage / 1 d
and
   t   is the time of the needed position / 1 d (both times MJD, e.g.).

it gives
   xe,ye,ze: the coordinates
   r:  distance of the object from the barycentre of the solar system

made by Rudolf Novak and J.Hollan}


procedure object_hel(a,q,exc,i,AN,PN,t0,t: real; var xe,ye,ze,r:real);
var M,e0,e1,e2,v,u,f0,f1,f2,adx,ady,af0,af1,m1od:real; n_it:byte;
const
 k=0.01720209895;  {Gauss gravitational constant}
 Elip:boolean=true;

 function sinh(x:real):real;
  begin
   sinh:=(exp(x)-exp(-x))/2;
  end;

 function Kepl_eli(e:real):real;
  begin
   Kepl_eli:=e-exc*sin(e)-M;
  end;

 function Kepl_hyp(e:real):real;
  begin
   Kepl_hyp:=-e+exc*sinh(e)-M;
  end;

begin
 if {exc<>1} abs(1-exc)>3E-6 then
  begin
   if exc>1 then Elip:=false;
   if a=0 then a:=q/(1-exc);
   M:=k/(sqrt(abs(a*a*a)))*(t-t0);
   M:=deg_to_rad(Bas_Int(rad_to_deg(M),360));
   if M>pi then M:=M-2*pi;
   e0:=M;
   if M>0 then
    e1:=exp(ln(M/6)/3)
   else
    e1:=-exp(ln(-M/6)/3);
   if Elip then
    begin
     f0:=Kepl_eli(e0);
     f1:=Kepl_eli(e1);
    end
   else
    begin
     f0:=Kepl_hyp(e0);
     f1:=Kepl_hyp(e1);
    end;
{   writeln(e0,' ',f0);
   writeln(e1,' ',f1);
 }  ady:=abs(f1-f0);
   adx:=abs(e1-e0);
   af1:=abs(f1);
   n_it:=0;
   while (adx>1E-10) and (af1>0) and (ady/af1>1E-10) do
          {iteration by "metoda secen"}
     begin
      m1od:=(e0-e1)/(f1-f0);
      if abs(f1)<abs(f0) then
       e2:=e1+f1*m1od
      else
       e2:=e0+f0*m1od;
      if Elip then
       f2:=Kepl_eli(e2)
      else
       f2:=Kepl_hyp(e2);
{      writeln(e2,' ',f2);
 }     e0:=e1;
      f0:=f1;
      e1:=e2;
      f1:=f2;
      adx:=abs(e1-e0);
      ady:=abs(f1-f0);
      af0:=af1;
      af1:=abs(f1);
      inc(n_it);
      if (n_it>7) and (af1>af0) and (af1/abs(e1)<2E-10) then
       begin
        af1:=0;
        e2:=e1;
       end;
     end;
   v:=2*arctan(sqrt(((1+exc)/abs((1-exc))))*(sin(e2/2)/cos(e2/2)));
   r:=a*(1-sqr(exc))/(1+exc*cos(v));
  end
 else
  begin
   M:=3*k/(q*sqrt(2*q))*(t-t0);
   e1:=M/3;
{   writeln(e1);
 }  repeat {iteration by Newton-Raphson method}
    e0:=e1;
    e2:=sqr(e0);
    e1:=e0+(M-e0*(e2+3))/(3*(e2+3));
{    writeln(e1);
 }  until (abs(e1-e0)<2E-10);
   v:=2*arctan(e1);
   r:=q*(1+sqr(e1));
  end;

{ writeln(v,r);
 }u:=PN + v;
 xe:=r*(cos(u)*cos(AN)-sin(u)*sin(AN)*cos(i));
 ye:=r*(cos(u)*sin(AN)+sin(u)*cos(AN)*cos(i));
 ze:=r*sin(u)*sin(i);
end;

function Heliocentric_Correction_s(jdat,RA_l,de_b:real):real;
 var l_e,b_e,r_e:real; V1,V2:v3; name_e:string;
begin
 if PrecInit(2000,2000+(JDat-JD2000)/365.25) then
  Precess(RA_l,de_b);
 ra_l:=deg_to_rad(ra_l);  de_b:=deg_to_rad(de_b);
 ra_d2l_b(epsi_rad,ra_l,de_b);
 Planet_Hel(3,jdat,l_e,b_e,r_e,name_e);
 l_e:=deg_to_rad(l_e);  b_e:=deg_to_rad(b_e);
 P3s_vector(l_e,b_e,V1);
 P3s_vector(ra_l,de_b,V2);
 Heliocentric_Correction_s:=scal_prod(V1,V2)*r_e*AU/speed_of_light;
end;

procedure Init_Moon_pole(JD_p:real);
var lp:real;
begin
 lp:=(JD_p-JD2000)/JulCentury;
 lp:=125.044522+lp*(
   -1934.136261+lp*(
       0.002071+lp*
       0.000002    ))
     +90;
 planet_pole[11,4]:=Bas_int(lp,360);
 {! has to be
 ra_d2l_b(-epsi_radJ2000,...}
 lb_rd(planet_pole[11,4],90-planet_pole[11,3],
       planet_pole[11,1],planet_pole[11,2]);
end;

procedure init_ver_eq;
var j:1..12; l, b, lo,bo,lv,bv:real;
begin
 for j:=1 to 11 do
  begin
   L:=deg_to_rad(planet_pole[j,1]);
   B:=deg_to_rad(planet_pole[j,2]);
   Lo:=planet_pole[j,3];
   Bo:=planet_pole[j,4];
   ra_d2l_b(epsi_rad,L,B);  {ecliptical coord. of planet's pole}
   if j=10 then
    begin
     L:=l+pi/2;      {ascending node of solar equator}
     B:=0;
    end
   else
    begin
     vect_prod_dir(l,b,
                   deg_to_rad(Bo-90),
                     deg_to_rad(90-Lo),
                   lv,bv);
     l:=lv; b:=bv;
    end;
   ra_d2l_b(-epsi_rad,L,B);   {L and B are now terrestrial equatorial
                 coordinates of
                 ascending node of the orbit of the planet
                 on it's equator}
   L:=L-deg_to_rad(planet_pole[j,1]-270);
                              {origin of L-coordinate has been rotated to the
                 ascending node of the equator of the planet
                 on the Earth equator}
   ra_d2l_b(deg_to_rad(90-planet_pole[j,2]),L,B);
                 {L is now measured along the equator of the planet
                 from its above-mentioned ascending node}
   planet_ver_eq[j]:=rad_to_deg(L);
  end;
end;

begin
 init_ver_eq; {initialization of true vernal equinoxes of the planets}
end.
