 {~ Fraction of light transmitted through a layer of glass at various angles~}

program Transmis; {by Jan Hollan}

const
cl=#13#10;
h1=cl+'Transmis [#] [n#]'+cl+cl+
'Per cent of light'+cl+
' which gets through a non-absorbing layer of glass';
h2=
' at a given angle of incidence (not considering multiple reflections).'+cl;
h3=cl+
'A single angle (/1 degree) may be given as a parameter,'+cl+
'otherwise a series of angles from 5 to 85 degrees are computed.'+cl+cl+
'N# gives another index or refraction than 1.54'+cl+cl;
h4=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;

 n_glass:real=1.54;
 Just_one_angle:boolean=false;

 wrong_code:integer=1;

var
I, PosN:byte;
Ai,AiD:real; {angle of incidence}
Ar:real;     {angle of refracted ray}
Sa:real;
Da:real;
S,T,F_perp_pl,F_in_pl,n_glass_n: real;

 procedure help;
  begin
   write(h1,cl,h2,h3,h4); halt;
  end;

 procedure One_Angle;
  begin
   if AiD<1E-3 then
    begin
     Ar:=0;
     F_perp_pl:=100*2/(n_glass+1/n_glass);
     F_in_pl  :=F_perp_pl;
    end
   else
    begin
     Ai:=pi*AiD/180;
     Ar:=sin(Ai)/n_glass;
     Ar:=arctan(Ar/sqrt(1-Sqr(Ar)));
      {Now, both Ai and Ar are in radians.}
     Sa:=Ai+Ar;
     Da:=Ai-Ar;
     S:=sqr(sin(Da)/sin(Sa));
     T:=sqr(sin(Da)*cos(Sa)/(sin(Sa)*cos(Da)));
     F_perp_pl:=100*(1-T)/(1+T);
     F_in_pl  :=100*(1-S)/(1+S);
    end;
   writeln(AiD:6:1,180*Ar/pi:8:1,F_perp_pl:7:1,F_in_pl:7:1,
           (F_perp_pl+F_in_pl)/2:8:1,
           100*(1-sqrt((F_perp_pl+F_in_pl)/200)):8:1);
  end;

begin
 if paramcount>0 then
  for I:=1 to paramcount do
   begin
    if pos('?',paramstr(i))>0 then help;
    PosN:=pos('N',paramstr(i));
    if PosN=0 then PosN:=pos('n',paramstr(i));
    if PosN>0 then
     begin
      val(copy(paramstr(i),PosN+1,length(paramstr(i))-PosN),n_glass_n,wrong_code);
      if wrong_code=0 then n_glass:=n_glass_n;
     end
    else
     begin
      val(paramstr(i),AiD,wrong_code);
      Just_One_angle:= (wrong_code=0) and (AiD<90) and (AiD>=0);
     end;
   end;
 writeln(h1,' with n=',n_glass:5:3,cl,h2);
 if Just_One_angle then writeln;
 writeln(
      '   angles            transmitted        reflected'+cl+
      ' / 1 degree     electric vector         at each'+cl+
      'outside inside   perp. along   average  surface');
 if Just_One_angle then
  begin
   writeln;
   One_Angle;
  end
 else
  for i:=0 to 17 do
   begin
    AiD:=i*5;
    One_Angle;
   end;
end.
